subroutine driver_stability2(ybb, pbb, iparam, rparam) 

!	This subroutine calls some IMSL functions (DUMINF) to check for phase
!	stability (via minimum tangent plane distance) in case the Michelsen
!	routine comes up empty handed.
!
!	Record of revisions:
!			Date		Programmer				Description of Change
!			====		==========				=====================
!			3/31/00		P.D. Ting				inception
!
use essentials
implicit none

!	Declaring input/output argument list
integer, dimension(7), intent(in) :: IPARAM
real (KIND=double), dimension(7), intent(in) :: RPARAM
real (KIND=double), intent(inout) :: ybb
real (KIND=double), dimension(ncp), intent(inout) :: pbb

!	Declaring IMSL variables
real (KIND=double), dimension(ncp) :: XSCALE
real (KIND=double) :: FSCALE
integer :: iersvr, ipact, isact

!	Local temporary variables
real (KIND=double) :: sum, sum2
real (KIND=double), dimension(ncp) :: xguess_uminf, x_uminf
real (KIND=double) :: fvalue_uminf
integer :: i,k, j

external minf_fcn


!	Conditioning the IMSL parameters
forall (i=1:ncp)
	XSCALE(i) = 1.0d0
end forall
FSCALE = 1.0d0
iersvr = 0
ipact = 0
isact = 0


sum = 0.0d0

do i = 1, ncp
	xguess_uminf(i) = 1.0d0 / ncp
end do


call erset(iersvr, ipact, isact)
call DUMINF(minf_fcn, ncp, xguess_uminf, XSCALE, FSCALE, IPARAM, & 
	&	RPARAM, x_uminf, fvalue_uminf)
	
if (fvalue_uminf .lt. ybb) then 
  ybb = fvalue_uminf

  sum2 = 0.0d0
  do k = 1,ncp
    sum2 = sum2 + x_uminf(k)
  end do
  do k = 1, ncp
!	pbb(k) = x_uminf(k)/sum2
    pbb(k) = x_uminf(k)
 
  end do

end if



!	Repeat DUMINF with a new initial guess.  This initial guess favors 
!	the higher end of the composition scale.

if (ybb .gt. -1.0d-17) then
  do i = 1, ncp
	sum = 1.0d-6 / (ncp - 1)
	xguess_uminf(i) = 0.999999

	forall (j=1:ncp, j .ne. i) 
	  xguess_uminf(j) = sum
	end forall

	call DUMINF(minf_fcn, ncp, xguess_uminf, XSCALE, FSCALE, IPARAM, RPARAM, x_uminf,&
			& fvalue_uminf)

	if (fvalue_uminf .lt. ybb) then 
	  ybb = fvalue_uminf
      sum2 = 0.0d0
	  do k = 1,ncp
		sum2 = sum2 + x_uminf(k)
	  end do
	  do k = 1, ncp
!		pbb(k) = x_uminf(k)/sum2
    pbb(k) = x_uminf(k)

	  end do
	end if
  end do
end if	


!	Repeat the calculation with a 3rd set of input parameters
if (ybb .gt. -1.0d-17) then
  do i = 1, ncp
	sum = 0.15 / (ncp - 1)
	xguess_uminf(i) = 0.85

	forall (j=1:ncp, j .ne. i) 
	  xguess_uminf(j) = sum
	end forall

	call DUMINF(minf_fcn, ncp, xguess_uminf, XSCALE, FSCALE, IPARAM, RPARAM, x_uminf,&
			& fvalue_uminf)

	if (fvalue_uminf .lt. ybb) then 
	  ybb = fvalue_uminf

      sum2 = 0.0d0

	  do k = 1,ncp
		sum2 = sum2 + x_uminf(k)
	  end do
	  do k = 1, ncp
!		pbb(k) = x_uminf(k)/sum2
    pbb(k) = x_uminf(k)

	  end do


	end if
  end do
end if	


!	Repeat the calculations with a 4th set of input parameters
if (ybb .gt. -1.0d-17) then
  do i = 1, ncp
	sum = 0.0001 / (ncp - 1)
	xguess_uminf(i) = 0.9999

	forall (j=1:ncp, j .ne. i) 
	  xguess_uminf(j) = sum
	end forall

	call DUMINF(minf_fcn, ncp, xguess_uminf, XSCALE, FSCALE, IPARAM, RPARAM, x_uminf,&
			& fvalue_uminf)

	if (fvalue_uminf .lt. ybb) then 
	  ybb = fvalue_uminf
      sum2 = 0.0d0
	  do k = 1,ncp
		sum2 = sum2 + x_uminf(k)
	  end do
	  do k = 1, ncp
!		pbb(k) = x_uminf(k)/sum2
    pbb(k) = x_uminf(k)

	  end do
	end if
  end do
end if	


!	Repeat the calculation with a 5th set of input parameters
if (ybb .gt. -1.0d-17) then
  do i = 1, ncp
	sum = 0.00011 / (ncp - 1)
	xguess_uminf(i) = 0.99989

	forall (j=1:ncp, j .ne. i) 
	  xguess_uminf(j) = sum
	end forall

	call DUMINF(minf_fcn, ncp, xguess_uminf, XSCALE, FSCALE, IPARAM, RPARAM, x_uminf,&
			& fvalue_uminf)

	if (fvalue_uminf .lt. ybb) then 
	  ybb = fvalue_uminf
      sum2 = 0.0d0
	  do k = 1,ncp
		sum2 = sum2 + x_uminf(k)
	  end do
	  do k = 1, ncp
!		pbb(k) = x_uminf(k)/sum2
    pbb(k) = x_uminf(k)

	  end do
	end if
  end do
end if	

!	Repeat the calculation with a 6th set of input parameters

if (ybb .gt. -1.0d-17) then
  do i = 1, ncp
	sum = 0.00009 / (ncp - 1)
	xguess_uminf(i) = 0.99991

	forall (j=1:ncp, j .ne. i) 
	  xguess_uminf(j) = sum
	end forall

	call DUMINF(minf_fcn, ncp, xguess_uminf, XSCALE, FSCALE, IPARAM, RPARAM, x_uminf,&
			& fvalue_uminf)

	if (fvalue_uminf .lt. ybb) then 
	  ybb = fvalue_uminf
      sum2 = 0.0d0
	  do k = 1,ncp
		sum2 = sum2 + x_uminf(k)
	  end do
	  do k = 1, ncp
!		pbb(k) = x_uminf(k)/sum2
    pbb(k) = x_uminf(k)

	  end do
	end if
  end do
end if	

	
!	Repeat the calculation with a 7th set of input parameters

if (ybb .gt. -1.0d-17) then
  do i = 1, ncp
    sum = 0.05 / (ncp - 1)
    xguess_uminf(i) = 0.95

    forall (j=1:ncp, j .ne. i) 
	  xguess_uminf(j) = sum
	end forall

	call DUMINF(minf_fcn, ncp, xguess_uminf, XSCALE, FSCALE, IPARAM, RPARAM, x_uminf,&
			& fvalue_uminf)

	if (fvalue_uminf .lt. ybb) then 
	  ybb = fvalue_uminf
      sum2 = 0.0d0
	  do k = 1,ncp
		sum2 = sum2 + x_uminf(k)
	  end do
	  do k = 1, ncp
!		pbb(k) = x_uminf(k)/sum2
    pbb(k) = x_uminf(k)

	  end do
    end if
  end do
end if	


!	Repeat the calculation with a 7th set of input parameters


if (ybb .gt. -1.0d-17) then
  do i = 1, ncp
    sum = 0.2 / (ncp - 1)
	xguess_uminf(i) = 0.8
	forall (j=1:ncp, j .ne. i) 
	  xguess_uminf(j) = sum
	end forall

	call DUMINF(minf_fcn, ncp, xguess_uminf, XSCALE, FSCALE, IPARAM, RPARAM, x_uminf,&
			& fvalue_uminf)

	if (fvalue_uminf .lt. ybb) then 
	  ybb = fvalue_uminf
      sum2 = 0.0d0
	  do k = 1,ncp
		sum2 = sum2 + x_uminf(k)
	  end do
	  do k = 1, ncp
!		pbb(k) = x_uminf(k)/sum2
    pbb(k) = x_uminf(k)

	  end do
	end if
  end do
end if
		

!	Repeat the calculation with a 8th set of input parameters

if (ybb .gt. -1.0d-17) then
  do i = 1, ncp
	sum = 0.25 / (ncp - 1)
	xguess_uminf(i) = 0.75
    forall (j=1:ncp, j .ne. i) 
	  xguess_uminf(j) = sum
	end forall

	call DUMINF(minf_fcn, ncp, xguess_uminf, XSCALE, FSCALE, IPARAM, RPARAM, x_uminf,&
			& fvalue_uminf)

	if (fvalue_uminf .lt. ybb) then 
	  ybb = fvalue_uminf
      sum2 = 0.0d0
	  do k = 1,ncp
		sum2 = sum2 + x_uminf(k)
	  end do
	  do k = 1, ncp
!		pbb(k) = x_uminf(k)/sum2
    pbb(k) = x_uminf(k)

	  end do
	end if
  end do
end if	

!	Repeat the calculation with a 9th set of input parameters

if (ybb .gt. -1.0d-17) then
  do i = 1, ncp
	sum = 0.3 / (ncp - 1)
	xguess_uminf(i) = 0.7
	forall (j=1:ncp, j .ne. i) 
	  xguess_uminf(j) = sum
	end forall

	call erset(iersvr, ipact, isact)
	call DUMINF(minf_fcn, ncp, xguess_uminf, XSCALE, FSCALE, IPARAM, RPARAM, x_uminf,&
		& fvalue_uminf)

	if (fvalue_uminf .lt. ybb) then 
	  ybb = fvalue_uminf
      sum2 = 0.0d0
	  do k = 1,ncp
		sum2 = sum2 + x_uminf(k)
	  end do
	  do k = 1, ncp
!		pbb(k) = x_uminf(k)/sum2
    pbb(k) = x_uminf(k)

	  end do
	end if
  end do
end if	


!	Repeat the calculation with the 10th set of input parameters
if (ybb .gt. -1.0d-17) then
  do i = 1, ncp
    sum = 0.35 / (ncp - 1)
	xguess_uminf(i) = 0.65
	forall (j=1:ncp, j .ne. i) 
	  xguess_uminf(j) = sum
	end forall

	call DUMINF(minf_fcn, ncp, xguess_uminf, XSCALE, FSCALE, IPARAM, RPARAM, x_uminf,&
			& fvalue_uminf)

	if (fvalue_uminf .lt. ybb) then 
	  ybb = fvalue_uminf
      sum2 = 0.0d0
	  do k = 1,ncp
		sum2 = sum2 + x_uminf(k)
	  end do
	  do k = 1, ncp
!		pbb(k) = x_uminf(k)/sum2
    pbb(k) = x_uminf(k)

	  end do
	end if
  end do
end if	

!	Repeat the calculation with a 11th set of input parameters

if (ybb .gt. -1.0d-17) then
  do i = 1, ncp
    sum = 0.5 / (ncp - 1)
	xguess_uminf(i) = 0.5
	forall (j=1:ncp, j .ne. i) 
	  xguess_uminf(j) = sum
	end forall

    call DUMINF(minf_fcn, ncp, xguess_uminf, XSCALE, FSCALE, IPARAM, RPARAM, x_uminf,&
			& fvalue_uminf)

	if (fvalue_uminf .lt. ybb) then 
	  ybb = fvalue_uminf
      sum2 = 0.0d0
	  do k = 1,ncp
		sum2 = sum2 + x_uminf(k)
	  end do
	  do k = 1, ncp
!		pbb(k) = x_uminf(k)/sum2
    pbb(k) = x_uminf(k)

	  end do
	end if
  end do
end if	


	if (ybb .gt. -1.0d-17) then
		do i = 1, ncp
			sum = 0.6 / (ncp - 1)
			xguess_uminf(i) = 0.4
			forall (j=1:ncp, j .ne. i) 
				xguess_uminf(j) = sum
			end forall

			call DUMINF(minf_fcn, ncp, xguess_uminf, XSCALE, FSCALE, IPARAM, RPARAM, x_uminf,&
					& fvalue_uminf)

			if (fvalue_uminf .lt. ybb) then 
				ybb = fvalue_uminf
      sum2 = 0.0d0
	  do k = 1,ncp
		sum2 = sum2 + x_uminf(k)
	  end do
	  do k = 1, ncp
!		pbb(k) = x_uminf(k)/sum2
    pbb(k) = x_uminf(k)

	  end do
			end if
		end do
	end if	


	if (ybb .gt. -1.0d-17) then
		do i = 1, ncp
			sum = 0.7 / (ncp - 1)
			xguess_uminf(i) = 0.3
			forall (j=1:ncp, j .ne. i) 
				xguess_uminf(j) = sum
			end forall
			call DUMINF(minf_fcn, ncp, xguess_uminf, XSCALE, FSCALE, IPARAM, RPARAM, x_uminf,&
					& fvalue_uminf)

			if (fvalue_uminf .lt. ybb) then 
				ybb = fvalue_uminf
      sum2 = 0.0d0
	  do k = 1,ncp
		sum2 = sum2 + x_uminf(k)
	  end do
	  do k = 1, ncp
!		pbb(k) = x_uminf(k)/sum2
    pbb(k) = x_uminf(k)

	  end do
			end if
		end do
	end if	

	if (ybb .gt. -1.0d-17) then
		do i = 1, ncp
			sum = 0.8 / (ncp - 1)
			xguess_uminf(i) = 0.2
			forall (j=1:ncp, j .ne. i) 
				xguess_uminf(j) = sum
			end forall

			call DUMINF(minf_fcn, ncp, xguess_uminf, XSCALE, FSCALE, IPARAM, RPARAM, x_uminf,&
					& fvalue_uminf)

			if (fvalue_uminf .lt. ybb) then 
				ybb = fvalue_uminf
      sum2 = 0.0d0
	  do k = 1,ncp
		sum2 = sum2 + x_uminf(k)
	  end do
	  do k = 1, ncp
!		pbb(k) = x_uminf(k)/sum2
    pbb(k) = x_uminf(k)

	  end do
			end if
		end do
	end if	


	if (ybb .gt. -1.0d-17) then
		do i = 1, ncp
			sum = (1-0.87) / (ncp - 1)
			xguess_uminf(i) = 0.87
			forall (j=1:ncp, j .ne. i) 
				xguess_uminf(j) = sum
			end forall

			call DUMINF(minf_fcn, ncp, xguess_uminf, XSCALE, FSCALE, IPARAM, RPARAM, x_uminf,&
					& fvalue_uminf)

			if (fvalue_uminf .lt. ybb) then 
				ybb = fvalue_uminf
      sum2 = 0.0d0
	  do k = 1,ncp
		sum2 = sum2 + x_uminf(k)
	  end do
	  do k = 1, ncp
!		pbb(k) = x_uminf(k)/sum2
    pbb(k) = x_uminf(k)

	  end do
			end if
		end do
	end if	


	if (ybb .gt. -1.0d-17) then
		do i = 1, ncp
			sum = 0.9 / (ncp - 1)
			xguess_uminf(i) = 0.1
			forall (j=1:ncp, j .ne. i) 
				xguess_uminf(j) = sum
			end forall

			call DUMINF(minf_fcn, ncp, xguess_uminf, XSCALE, FSCALE, IPARAM, RPARAM, x_uminf,&
					& fvalue_uminf)

			if (fvalue_uminf .lt. ybb) then 
				ybb = fvalue_uminf
      sum2 = 0.0d0
	  do k = 1,ncp
		sum2 = sum2 + x_uminf(k)
	  end do
	  do k = 1, ncp
!		pbb(k) = x_uminf(k)/sum2
    pbb(k) = x_uminf(k)

	  end do
			end if
		end do
	end if
	
END SUBROUTINE	