double precision function gibbsfcn(x_in)

!	Takes a given composition set x_in(i)s.  First uses liquid and vapor like densities
!	to see which one has a lower total gibbs free energy corresponding to the specified 
!	temperature and pressure.  Then it calculates the gibbs free energy of the more stable
!	phase. The final output gibbsfcn is given as solutions to the equation:
!
!				gibbsfcn = delta_g = sum[ x(i)* (chpt_i(x) - chpt_i(z) ) ]
!
!	where chpt_i is the chemical potential of each component i and x and z are composition
!	vectors. 
!
!	Record of revisions:
!			Date		Programmer				Description of Change
!			====		==========				=====================
!			4/02/00		P.D. Ting				inception
!

use essentials
implicit none

!	Defining thermoprops variables
real (KIND=double), dimension(ncp) :: x_chpt, chpt, fugacity, chpt_res
real (KIND=double) :: rhox
real (KIND=double), dimension(ncp) :: m
real (KIND=double), dimension(ncp,ncp) :: u0
real (KIND=double), dimension(ncp) :: e_ii
real (KIND=double), dimension(ncp,ncp,max,max) :: kappa
real (KIND=double), dimension(ncp,ncp) :: sig
real (KIND=double), dimension(ncp) :: v00
real (KIND=double), dimension(ncp,ncp) :: kij
real (KIND=double), dimension(ncp) :: x, xd,xq,q,mu
real (KIND=double), dimension(ncp,ncp,max,max,2) :: fa
real (KIND=double), dimension(ncp,ncp,max,max,1) :: fb
integer, dimension(ncp) :: N_sites
real (KIND=double) :: Prsm, Ursm, Arsm, Grsm, Srsm, Hrsm, Pcalc, Gtotal

!	Defining internal variables
real (KIND=double), dimension(2) :: Gtotal_compare, rhox_compare, Pcalc_compare
real (KIND=double) :: sum
real (KIND=double), dimension(ncp) :: x_passaround
integer :: i

!	Defining input/output arguments
real (KIND=double), intent(in), dimension(ncp) :: x_in


forall (i=1:ncp)
  x_passaround(i) = x_tmp(i)
end forall

forall (i = 1:ncp)
  x_tmp(i) = x_in(i)
end forall

call dimension_convert(m, u0, e_ii, kappa, sig, v00, kij, x, fa, &
		&fb, N_sites,q,mu,xq,xd)


call liquid_or_vapor(rhox, x_in)

call PTthermo_props(ncp, max, status11, TK, rhox, m, x_in, u0, e_ii, kij, &
			& N_sites, fa, fb, kappa, sig, D, v00, Prsm, Ursm, Arsm, Grsm, &
			& Srsm, Hrsm, chpt, Pcalc, fugacity, chpt_res, Gtotal, pc_dispersion,&
			& analytic_flag, polar_flag, Q, mu, xd, xq) 

sum = 0.0d0

do i = 1, ncp
  sum = x_in(i) * (chpt(i) - z_chpt(i) ) + sum
end do

gibbsfcn = sum

do i = 1, ncp
x_tmp(i) = x_passaround(i)
end do

end function