Subroutine partition_setup(pseudoX, fugacity, phase)

!	This subroutine sets up the initial partition coefficient for use with Rachford-Rice
!	Calculations

!	Record of revisions:
!			Date		Programmer				Description of Change
!			====		==========				=====================
!			3/31/00		P.D. Ting				inception
!

use essentials
implicit none

real(kind=double), intent(out), dimension(ncp) :: pseudoX
real(kind=double), intent(in), dimension(ncp) :: fugacity
real (KIND=double) :: sum
integer, intent(in) :: phase
integer :: i 



!	Setup the initial partition coefficient K
forall (i=1:ncp)
  pseudoX(i) = dexp( h(i) - fugacity(i) )
end forall

do i = 1, ncp
  if (pseudoX(i) .lt. 1.0d-12) pseudoX(i) = 1.0d-12
end do


!	for a vapor feed
if (phase.eq.1) then
  forall (i=1:ncp)
    partition(i) = z(i) / pseudoX(i)
  end forall

!	for a liquid feed
else
  forall(i=1:ncp)
    partition(i) = pseudoX(i) / z(i)
  end forall

  sum = 0.0d0
  do i = 1, ncp
    sum = sum + z(i) * partition(i)
  end do
 
  forall (i=1:ncp)
    partition(i) = pseudoX(i) / z(i) * sum
  end forall

end if


END SUBROUTINE