subroutine test_multipolarmix(ncp, TK,rhox, zeta3, sig, m, x, dhs, mu, & 
& Q, xd, xq, sitos1, a_mp, z_mp, chpt_mp, g_mp)

Implicit none

! Subroutine multipolarmix to calculate the dipolar and quadripolar contribution
! to thermodynamic properties.
! Programmed by Auleen Ghosh on November 26, 2002
! Modified by Matt Yarrison October 2003
! Corrected to avoid problems when mole fraction of polar component goes to zero. 
! Check before running with systems that have more than one polar component
!	Checking single & double precision
integer, parameter :: single = SELECTED_REAL_KIND(p=6, r=37)
integer, parameter :: double = SELECTED_REAL_KIND(p=13)

integer, intent(in) :: ncp	! Number of components
integer :: i, j, k, n, p
!
real (KIND=double), intent(in) :: TK ! Temperature in Kelvin
real (KIND=double) :: rhox						! number density of molecules (per ang^3)
real (KIND=double), intent(in) :: zeta3	! Packing fraction
real (KIND=double), intent(in), dimension(ncp, ncp) :: sig  ! Temperature independent segment diameter
real (KIND=double), intent(in), dimension(ncp) :: m			! chain length of molecule i
real (KIND=double), intent(in), dimension(ncp) :: x			! mole fraction of component i
real (KIND=double), intent(in), dimension(ncp) :: dhs		! temp. dependent diameter
real (KIND=double), intent(in), dimension(ncp) :: Q		    ! quadrupole moment of component i in 
real (KIND=double), intent(in), dimension(ncp) :: mu		! Dipole moment of component i in debye
real (KIND=double), intent(in), dimension(ncp) :: sitos1	! dhs/sig(1,1)
!				   
real (KIND=double), dimension(0:5, 15) :: J2  ! 
real (KIND=double), dimension(0:5, 4):: JT
real (KIND=double), dimension(15)  :: I2       ! The integral and its density derivative used in second order multipolar perturbation of free energy
real (KIND=double), dimension(15)  :: dI_drsr  ! d(I2())/d(rhox)
real (KIND=double), dimension(4)   :: IT       ! The integral used in third order multipolar perturbation of free energy
real (KIND=double), dimension(4)   :: dIT_drsr ! d(IT())/d(rhox)
real (KIND=double), dimension(ncp) :: d_dx3_dx  ! d(dx^3)/dx(i)
real (KIND=double), dimension(ncp) :: drsr_dx   ! d(rho_segment_reduced)/dx(i)
real (KIND=double), dimension(ncp) :: x_dip		! fraction of dipolar segments in component i
real (KIND=double), dimension(ncp) :: x_qud		! fraction of quadrupolar segments in component i
real (KIND=double), dimension(ncp) :: x_mtp		! fraction of multipolar (di or quad) segments in component i
real (KIND=double), dimension(ncp) :: xd		! Fraction of dipolar segments in a component
real (KIND=double), dimension(ncp) :: xq		! Fraction of quadripolar segments in a component



real (KIND=double), intent(out) :: a_mp, z_mp, g_mp	! MultiPolar contribution to Helmholtz free energy, compressibility factor and Gibbs free energy
real (KIND=double), intent(out), dimension(ncp) :: chpt_mp ! Multipolar contribution to chemical potential
!
real (KIND=double), parameter :: Kb= (8.31441D0/6.022045D+23), PI=3.141592654d0	! Boltzmann constant, Pi

real (KIND=double) :: D
real (KIND=double) :: myres
real (KIND=double) :: rho			! molecular number density (#molecules/A^3)
real (KIND=double) :: rho_seg		! segment number density (#segment/A^3)
real (KIND=double) :: rsr			! reduced segment number density (rho_seg * dx3)
real (KIND=double) :: dhs_ij, dhs_ki  
real (KIND=double) :: dhs_jk
real (KIND=double) :: A2, A3A, A3B, A2_1,A2_2,A2_3,A3A_1,A3A_2,A3A_3,A3B_1,A3B_2,A3B_3,A3B_4
!
real (KIND=double), dimension(ncp) :: ddendx, dd_dx, dj6dx, dkdd_dx, dd3_dx

! contribution to rho*d(A/NKT)/d(rho)
real (KIND=double) :: drsr_drho		! d(rho_segment_reduced)/d(rho)
real (KIND=double) :: dA2_drho, dA3A_drho, dA3B_drho 
real (KIND=double) :: dA2_1_drho, dA2_2_drho, dA2_3_drho 
real (KIND=double) :: dA3A_3_drho
real (KIND=double) :: dA3B_1_drho, dA3B_2_drho, dA3B_3_drho, dA3B_4_drho

! composition derivatives of 'A_mp'
real (KIND=double), dimension(ncp) :: dA3B_1_dx, dA3B_2_dx, dA3B_3_dx, dA3B_4_dx, dA3B_dx	
real (KIND=double), dimension(ncp) :: dA3A_3_dx, dA3A_dx
real (KIND=double), dimension(ncp) :: dA2_1_dx, dA2_2_dx, dA2_3_dx, dA2_dx, dA_dx
real (KIND=double), dimension(ncp) :: mu_2, mu_4, mu_6, Q_2, Q_4, Q_6

real (KIND=double):: sum1, sum2, sum3, sum4
real (KIND=double):: dx3	! Van der waals 1-fluid mixing diameter cubed


! Kb=8.31441D0/6.022045D+23
! PI=3.141592654d0

!if (x(1).eq.0.0d0) then
!	a_mp=0.0d0
!	z_mp=0.0d0
!   g_mp=0.0d0
!end if
!return
	

do p=1,ncp

	!if ( (x(p).eq.1.0).or.(x(p).eq.0.0) ) then
	!	pause
	!end if

	rho=rhox/(sig(1,1)**3)
	D=zeta3/((PI/6.d0)*rho)

sum1 = 0.0d0
do i=1, ncp
	sum1 = sum1 + x(i)*m(i)
enddo

rho_seg = rho * sum1	! segment number density

!	1 debye = 1e-18 esu.cm; 1debye = 3.162 * 1e-25 (j/m^3)^0.5
!   unit of Q is 1e-26 esu.cm^2

do i=1, ncp
	mu_2(i) = mu(i)**2.d0 *1.d-49 /(Tk*kb*dhs(i)**3.d0*1.d-30)
	mu_4(i) = mu_2(i)*mu_2(i)
	mu_6(i) = mu_4(i)*mu_2(i)

	Q_2(i) = Q(i)**2.0d0*1d-69/(dhs(i)**5.0d0*kb*TK*1d-50) 
	Q_4(i) = Q_2(i)*Q_2(i)
	Q_6(i) = Q_4(i)*Q_2(i)
end do


J2(0,6)=4.1888d0
J2(1,6)=2.8287d0
J2(2,6)=0.8331d0
J2(3,6)=0.0317d0
J2(4,6)=0.0858d0
J2(5,6)=-0.0846d0

J2(0,8)=2.5133d0
J2(1,8)=2.1795d0
J2(2,8)=1.0423d0
J2(3,8)=0.2596d0
J2(4,8)=0.1097d0
J2(5,8)=-0.0573d0

J2(0,10)=1.7952d0
J2(1,10)=1.7551d0
J2(2,10)=1.0376d0
J2(3,10)=0.3890d0
J2(4,10)=0.1561d0
J2(5,10)=-0.0082d0

J2(0,11)=1.5708d0
J2(1,11)=1.5957d0
J2(2,11)=1.0079d0
J2(3,11)=0.4237d0
J2(4,11)=0.1740d0
J2(5,11)=0.0215d0

J2(0,13)=1.2566d0
J2(1,13)=1.347d0
J2(2,13)=0.9326d0
J2(3,13)=0.4562d0
J2(4,13)=0.1906d0
J2(5,13)=0.0865d0

J2(0,15)=1.0472d0
J2(1,15)=1.1631d0
J2(2,15)=0.8552d0
J2(3,15)=0.4506d0
J2(4,15)=0.1913d0
J2(5,15)=0.1465d0


!  JT: 1=TD, 2=DDQ, 3=DQQ, 4=TQ

JT(0,1)=16.4493d0
JT(1,1)=19.8096d0
JT(2,1)=6.3321d0
JT(3,1)=-0.0932d0
JT(4,1)=-1.1741d0
JT(5,1)=-0.8321d0

JT(0,2)=139.4906d0
JT(1,2)=241.9354d0
JT(2,2)=163.9581d0
JT(3,2)=57.0537d0
JT(4,2)=13.2686d0
JT(5,2)=-22.3827d0

JT(0,3)=139.4906d0
JT(1,3)=298.9225d0
JT(2,3)=283.4908d0
JT(3,3)=161.7222d0
JT(4,3)=81.7788d0
JT(5,3)=-35.8868d0

JT(0,4)=532.9586d0
JT(1,4)=1287.3491d0
JT(2,4)=1447.9762d0
JT(3,4)=1026.3847d0
JT(4,4)=608.4235d0
JT(5,4)=-20.5065d0

!  IT: IT(1)=I_TD, IT(2)=I_DDQ, IT(3)=I_DQQ, IT(4)=I_TQ
! 6,8,10,11,13,15

!------------------------------dx3 = VDW 1 fluid diameter cubed-------------------------
sum1 = 0.0d0
sum2 = 0.0d0

do i=1, ncp
	do j=1, ncp

		dhs_ij = 0.5d0*(dhs(i) + dhs(j))

		sum1 = sum1 + x(i)*x(j)*m(i)*m(j)*dhs_ij**3.0d0
		sum2 = sum2 + x(i)*x(j)*m(i)*m(j)

	enddo
enddo

dx3 = sum1 / sum2

!-----------------------------d(dx^3)/d(x(i))--------------------------------------------
sum3 = 0.0d0
sum4 = 0.0d0

do i=1, ncp

	d_dx3_dx(i) = 0.0d0

	do j=1, ncp

		dhs_ij = 0.5d0*(dhs(i) + dhs(j))

		sum3 = sum3 + x(j)*m(j)*dhs_ij**3.0d0
		sum4 = sum4 + x(j)*m(j)

	enddo

	d_dx3_dx(i) = m(i) * 2.0d0 * (sum3 - dx3 * sum4) / sum2

enddo

!------------------------------------------------------------------------------------------

rsr = rho_seg * dx3	! RHO_SEG_RED - REDUCED SEGMENT NUMBER DENSITY


do i=1, ncp
	drsr_dx(i) = rho * m(i) * dx3 + rho_seg * d_dx3_dx(i)
enddo

do i=1,4

if (i.eq.1) then
n=6
else if (i.eq.2) then
n=8
else if (i.eq.3) then
n=10
else if (i.eq.4) then
n=15
end if

I2(n) = J2(0,n) + J2(1,n) * rsr + J2(2,n) * rsr**2.0d0 + &
		& J2(3,n) * rsr**3.0d0 + J2(4,n) * rsr**4.0d0 + J2(5,n) * rsr**5.0d0

dI_drsr(n)=J2(1,n) + 2.0d0 * J2(2,n)*rsr + 3.0d0 * J2(3,n) * rsr**2.0d0 + &
		&	4.0d0 * J2(4,n) * rsr**3.0d0 + 5.0d0 * J2(5,n)*rsr**4.0d0

enddo

do n=1,4

IT(n) = JT(0,n) + JT(1,n) * rsr + JT(2,n) * rsr**2.0d0 + JT(3,n) * rsr**3.0d0 + &
		&	JT(4,n) * rsr**4.0d0 + JT(5,n) * rsr**5.0d0

dIT_drsr(n) = JT(1,n) + 2.0d0 * JT(2,n) * rsr + 3.0d0 * JT(3,n) * rsr**2.0d0 + &
		&	4.0d0 * JT(4,n) * rsr**3.0d0 + 5.0d0 * JT(5,n) * rsr**4.0d0
enddo

drsr_drho = (rsr / rho) 

! Pure component

!A2 = -rhox * ( mu_4*I2(6)/6.0d0 + mu_2*Q_2*I2(8)/2.0d0 + 7.0d0*Q_4*I2(10)/10.0d0 )

!A3A = rhox * ( 2.0d0*mu_4*Q_2*I2(11)/5.0d0 + 12.0d0*mu_2*Q_4*I2(13)/35.0d0 + 36.0d0*Q_6*I2(15)/245.0d0 ) 

!A3B = rhox**2.0d0 * (mu_6*IT(1)/54.0d0 +mu_4*Q_2*IT(2)/480.0d0 + mu_2*Q_4*IT(3)/640.0d0 + Q_6*IT(4)/6400.0d0 )



! Mixtures


do i=1, ncp
	x_dip(i) = x(i) * m(i) * xd(i)
	x_qud(i) = x(i) * m(i) * xq(i)
enddo

A2=0.0d0
A2_1=0.0d0
A2_2=0.0d0
A2_3=0.0d0

do i=1,ncp
do j=1,ncp

dhs_ij = 0.5d0 * (dhs(i) + dhs(j))


A2_1 = A2_1 + rho * x_dip(i) * x_dip(j) * mu_2(i) * mu_2(j) * dhs(i)**3.0d0 * dhs(j)**3.0d0 &
		&	/ (dhs_ij)**3.0d0

A2_2 = A2_2 + rho * x_dip(i) * x_qud(j) * mu_2(i) * Q_2(j) * dhs(i)**3.0d0 * dhs(j)**5.0d0 & 
		&	/ (dhs_ij)**5.0d0

A2_3 = A2_3 + rho * x_qud(i) * x_qud(j) * Q_2(i) * Q_2(j) * dhs(i)**5.0d0 * dhs(j)**5.0d0 &
		&	/ (dhs_ij)**7.0d0

enddo
enddo

A2_1 = A2_1*I2(6)/6.0d0
A2_2 = A2_2*I2(8)/2.0d0
A2_3 = A2_3*I2(10)*7.0d0/10.0d0

A2 = - (A2_1 + A2_2 +A2_3)

! It is assumed that a molecule only has a dipole or a quadrupole, but not both.

!do i=1, ncp
!	if ((xq(i).le.0.0d0).and.(xd(i).ne.0.0d0)) then
!		x_mtp(i) = x(i) * m(i) * xd(i)
!	else if ((xd(i).le.0.0d0).and.(xq(i).ne.0.0d0)) then	
!		x_mtp(i) = x(i) * m(i) * xq(i)
!	end if
!end do


A3A_3=0.0d0
A3A=0.0d0

do i=1, ncp
do j=1, ncp

dhs_ij = 0.5d0 * (dhs(i) + dhs(j))


!A3A_1 = A3A_1 + rhox * x_mtp(i) * x_mtp(j) * mu_2(i) * mu_2(j) * (Q_2(i) * Q_2(j))**0.5d0 & 
!		& * (dhs(i) * dhs(j))**(11.0d0/2.0d0)/(dhs_ij)**11.0d0


!A3A_2 = A3A_2 + rhox * x_mtp(i) * x_mtp(j) * mu_2(i) * Q_2(i)**0.5d0 * Q_2(j)**1.5d0 & 
!		& * dhs(i)**(11.0d0/2.0d0) * dhs(j)**(15.0d0/2.0d0)/(dhs_ij)**13.0d0


A3A_3 = A3A_3 + rho * x_qud(i) * x_qud(j) * Q_2(i)**1.5d0 * Q_2(j)**1.5d0 & 
		& * dhs(i)**(15.0d0/2.0d0) * dhs(j)**(15.0d0/2.0d0)/(dhs_ij)**12.0d0

end do
end do

!A3A_1 = A3A_1 * I(11) * 2.0d0 / 5.0d0
!A3A_2 = A3A_2 * I(13) * 12.0d0 / 35.0d0

A3A_3 = A3A_3 * I2(15) * 36.0d0 / 245.0d0

!A3A = A3A_1 + A3A_2 + A3A_3

A3A = A3A_3
 

A3B_1 = 0.0d0
A3B_2 = 0.0d0
A3B_3 = 0.0d0
A3B_4 = 0.0d0
A3B   = 0.0d0


do i=1, ncp
do j=1, ncp
do k=1, ncp

dhs_ij = 0.5d0*(dhs(i) + dhs(j))
dhs_jk = 0.5d0*(dhs(j) + dhs(k))
dhs_ki = 0.5d0*(dhs(k) + dhs(i))



A3B_1 = A3B_1 + rho**2.0d0 * x_dip(i) * x_dip(j) * x_dip(k) * mu_2(i) * mu_2(j) * mu_2(k) * (dhs(i)*dhs(j)*dhs(k))**3.0d0 / &
		& (dhs_ij * dhs_jk * dhs_ki)

A3B_2 = A3B_2 + rho**2.0d0 * x_dip(i) * x_dip(j) * x_qud(k) * mu_2(i) * mu_2(j) * Q_2(k) * dhs(i)**3.0d0 * dhs(j)**3.0d0 &
		&	* dhs(k)**5.0d0 / (dhs_ij * dhs_jk**2.0d0 * dhs_ki**2.0d0)
		
A3B_3 = A3B_3 + rho**2.0d0 * x_dip(i) * x_qud(j) * x_qud(k) * mu_2(i) * Q_2(j) * Q_2(k) * &
		&	dhs(i)**3.0d0 * (dhs(j) * dhs(k))**5.0d0 / ((dhs_ij * dhs_ki)**2.0d0 * dhs_jk**3.0d0)
		
A3B_4 = A3B_4 + rho**2.0d0 * x_qud(i) * x_qud(j) * x_qud(k) * Q_2(i) * Q_2(j) * Q_2(k) * (dhs(i)*dhs(j)*dhs(k))**5.0d0 &
		&	/ (dhs_ij * dhs_jk * dhs_ki)**3.0d0

enddo
enddo
enddo

A3B_1 = A3B_1 * IT(1) / 54.0d0
A3B_2 = A3B_2 * IT(2) / 480.0d0
A3B_3 = A3B_3 * IT(3) / 640.0d0
A3B_4 = A3B_4 * IT(4) / 6400.0d0

A3B = A3B_1 + A3B_2 + A3B_3 + A3B_4


a_mp = A2**2.0d0/(A2 - A3A -A3B)		! dipole-quadrupole contribution to
if ( (A2-A3A-A3B).eq.0.0) then
	a_mp =0.0
end if										! mixture helmholtz energy A/NkT


!! -----------------density derivative of A--------------------------------------------------

!	dA2_1_drho = rho*d(A2_1)/d(rho)

dA2_1_drho = A2_1 + A2_1 * rho * dI_drsr(6) * drsr_drho / I2(6)
dA2_2_drho = A2_2 + A2_2 * rho * dI_drsr(8) * drsr_drho/ I2(8)
dA2_3_drho = A2_3 + A2_3 * rho * dI_drsr(10)* drsr_drho/ I2(10)


dA2_drho = - (dA2_1_drho + dA2_2_drho + dA2_3_drho)


dA3A_drho = A3A_3	+ A3A_3 * rho * dI_drsr(15) * drsr_drho / I2(15)

!	dA3B_1_drho = rhox*d(A3B_1)/d(rhox)

dA3B_1_drho = 2.0d0*A3B_1 + A3B_1 * rho * dIT_drsr(1) * drsr_drho / IT(1)
dA3B_2_drho = 2.0d0*A3B_2 + A3B_2 * rho * dIT_drsr(2) * drsr_drho / IT(2)
dA3B_3_drho = 2.0d0*A3B_3 + A3B_3 * rho * dIT_drsr(3) * drsr_drho / IT(3)
dA3B_4_drho = 2.0d0*A3B_4 + A3B_4 * rho * dIT_drsr(4) * drsr_drho / IT(4)

dA3B_drho =dA3B_1_drho + dA3B_2_drho + dA3B_3_drho +dA3B_4_drho


z_mp = (2.0d0*a_mp/A2) * dA2_drho - (a_mp/A2)**2.0d0 * (dA2_drho - dA3A_drho - dA3B_drho) 

if (A2.eq.0.0) then
	z_mp=0.0
end if

!! ------------------------------calculate dA/dx(i)----------------------------------------

!------------------------------------d(A2)/d(x(i))-----------------------------------------
do i=1,ncp

dA2_1_dx(i) =0.0d0
dA2_2_dx(i) =0.0d0
dA2_3_dx(i) =0.0d0

do j=1,ncp

dhs_ij = 0.5d0 * (dhs(i) + dhs(j))


dA2_1_dx(i) = dA2_1_dx(i) + 2.0d0 * rho * xd(i) * m(i) * x_dip(j) * mu_2(i) * mu_2(j) * dhs(i)**3.0d0 * dhs(j)**3.0d0 &
		&	/ (dhs_ij)**3.0d0

dA2_2_dx(i) = dA2_2_dx(i) + 2.0d0 * rho * xd(i) * m(i) * x_qud(j) * mu_2(i) * Q_2(j) * dhs(i)**3.0d0 * dhs(j)**5.0d0 &
		&	/ (dhs_ij)**5.0d0

dA2_3_dx(i) = dA2_3_dx(i) + 2.0d0 * rho * xq(i) * m(i) * x_qud(j) * Q_2(i) * Q_2(j) * dhs(i)**5.0d0 * dhs(j)**5.0d0 &
		&	/ (dhs_ij)**7.0d0

enddo
enddo

do i=1, ncp

	dA2_1_dx(i) = (dA2_1_dx(i) * I2(6) + A2_1 * dI_drsr(6) * drsr_dx(i)/I2(6)) / 6.0d0
	dA2_2_dx(i) = (dA2_2_dx(i) * I2(8) + A2_2 * dI_drsr(8) * drsr_dx(i)/I2(8))/ 2.0d0
	dA2_3_dx(i) = (dA2_3_dx(i) * I2(10)+ A2_3 * dI_drsr(10)* drsr_dx(i)/I2(10))* 7.0d0 / 10.0d0

	dA2_dx(i) = - (dA2_1_dx(i) + dA2_2_dx(i) + dA2_3_dx(i))

end do


!-------------------------------d(A3A)/d(x(i))-----------------------------------------------

do i=1, ncp

dA3A_3_dx(i) = 0.0d0

do j=1, ncp

dhs_ij = 0.5d0 * (dhs(i) + dhs(j))



dA3A_3_dx(i) = dA3A_3_dx(i) + 2.0d0 * rho * xq(i) * m(i) * x_qud(j) * Q_2(i)**1.5d0 * Q_2(j)**1.5d0 & 
		& * dhs(i)**(15.0d0/2.0d0) * dhs(j)**(15.0d0/2.0d0)/(dhs_ij)**12.0d0

end do
end do

do i=1, ncp
	dA3A_3_dx(i) = (dA3A_3_dx(i) * I2(15) + A3A_3 * dI_drsr(15) * drsr_dx(i)/I2(15))* 36.0d0 / 245.0d0
	dA3A_dx(i) = dA3A_3_dx(i)
enddo


!
!-------------------------------d(A3B)/d(x(i))---------------------------------------------


do i=1, ncp

dA3B_1_dx(i) = 0.0d0
dA3B_2_dx(i) = 0.0d0
dA3B_3_dx(i) = 0.0d0
dA3B_4_dx(i) = 0.0d0

do j=1, ncp
do k=1, ncp

dhs_ij = 0.5d0*(dhs(i) + dhs(j))
dhs_jk = 0.5d0*(dhs(j) + dhs(k))
dhs_ki = 0.5d0*(dhs(k) + dhs(i))



dA3B_1_dx(i) = dA3B_1_dx(i) + 3.0d0 * rho**2.0d0 * xd(i) * m(i) * x_dip(j) * x_dip(k) * mu_2(i) * mu_2(j) * mu_2(k) * (dhs(i)*dhs(j)*dhs(k))**3.0d0 / &
		& (dhs_ij * dhs_jk * dhs_ki) 

dA3B_2_dx(i) = dA3B_2_dx(i) + 3.0d0 * rho**2.0d0 * xd(i) * m(i) * x_dip(j) * x_qud(k) * mu_2(i) * mu_2(j) * Q_2(k) * dhs(i)**3.0d0 * dhs(j)**3.0d0 &
		&	* dhs(k)**5.0d0 / (dhs_ij * dhs_jk**2.0d0 * dhs_ki**2.0d0)
		
dA3B_3_dx(i) = dA3B_3_dx(i) + 3.0d0 * rho**2.0d0 * xd(i) * m(i) * x_qud(j) * x_qud(k) * mu_2(i) * Q_2(j) * Q_2(k) * &
		&	dhs(i)**3.0d0 * (dhs(j) * dhs(k))**5.0d0 / ((dhs_ij * dhs_ki)**2.0d0 * dhs_jk**3.0d0)
		
dA3B_4_dx(i) = dA3B_4_dx(i) + 3.0d0 * rho**2.0d0 * xq(i) * m(i) * x_qud(j) * x_qud(k) * Q_2(i) * Q_2(j) * Q_2(k) * (dhs(i)*dhs(j)*dhs(k))**5.0d0 &
		&	/ (dhs_ij * dhs_jk * dhs_ki)**3.0d0

enddo
enddo
enddo

do i=1, ncp
	dA3B_1_dx(i) = (dA3B_1_dx(i) * IT(1) + A3B_1 * dIT_drsr(1) * drsr_dx(i)/IT(1))/ 54.0d0
	dA3B_2_dx(i) = (dA3B_2_dx(i) * IT(2) + A3B_2 * dIT_drsr(2) * drsr_dx(i)/IT(2)) / 480.0d0
	dA3B_3_dx(i) = (dA3B_3_dx(i) * IT(3) + A3B_3 * dIT_drsr(3) * drsr_dx(i)/IT(3)) / 640.0d0
	dA3B_4_dx(i) = (dA3B_4_dx(i) * IT(4) + A3B_4 * dIT_drsr(4) * drsr_dx(i)/IT(4)) / 6400.0d0

	dA3B_dx(i) = dA3B_1_dx(i) + dA3B_2_dx(i) + dA3B_3_dx(i) + dA3B_4_dx(i)
enddo


do i=1, ncp
dA_dx(i) = (2.0d0*a_mp/A2) * dA2_dx(i) - (a_mp/A2)**2.0d0 * (dA2_dx(i) - dA3A_dx(i) - dA3B_dx(i)) 
enddo

myres=0.0d0
do i=1, ncp
	myres = myres - x(i) * dA_dx(i)
enddo

do i=1,ncp
      chpt_mp(i) = myres + a_mp + z_mp + dA_dx(i)
		if (((a_mp).eq.0.0).and.(z_mp.eq.0.0)) then
			chpt_mp(i) = 0.0
    	endif
enddo



!do i=1,ncp
!	if ((mu(i).ne.0).and.(x(i).eq.0.0d0)) then
!		chpt_mp(i) =0.0 
!		a_mp=0.0
!		z_mp=0.0
!	end if
!end do


g_mp = a_mp + z_mp



!write(6,*)'A2_1  = ',A2_1
!write(6,*)'A2_2_  = ',A2_2
!write(6,*)'A2_3  = ',A2_3
!write(6,*)'A2    = ',A2
!write(6,*)'A3A_3 = ',A3A_3
!write(6,*)'A3A   = ',A3A
!write(6,*)'A3B_1 = ',A3B_1
!write(6,*)'A3B_2_ = ',A3B_2
!write(6,*)'A3B_3_ = ',A3B_3
!write(6,*)'A3B_4 = ',A3B_4
!write(6,*)'A3B   = ',A3B

end do

return



end subroutine
