subroutine association(TK,rhox,eta,m,sigc,kappa,epsilon,lambda,a_assoc,z_assoc,num_assoc_site)
implicit none

!	Checking single & double precision
integer, parameter :: single = SELECTED_REAL_KIND(p=6, r=37)
integer, parameter :: double = SELECTED_REAL_KIND(p=13)

real (KIND=double) :: TK
real (KIND=double) :: eta,one_minus_eta,lambda !lambda=rcutoff/sigma
real (KIND=double) :: ghs, dg_deta,dg_drho
real (KIND=double) :: m,sigc,kappa,epsilon
real (KIND=double) :: integral,dint_drho,F_AB,I,deriv_I
real (KIND=double) :: delta_AB,ddelta_AB_drho,rho_delta_AB,drho_delta_AB_drho
real (KIND=double) :: rho_local,rhox
real (KIND=double) :: term_assoc1,term_assoc2
real (KIND=double) :: XA,dXA_drho
real (KIND=double) :: a_assoc,da_assoc_drho,z_assoc
integer :: correction, i_assoc_sites, num_assoc_site

!---if correction=0 then the integral is calculated with the Chapman approximation
!---if correction=1 then integral is corrected (Chang&Sandler)
correction=0
! Currently there is no mixing rule for the integral. Do not use to fit pure component parameters
!number of association sites
i_assoc_sites = num_assoc_site
! 

rho_local = rhox/sigc**3.0d0

one_minus_eta = 1.0d0-eta

ghs = (2.0d0-eta)/(2.0d0*one_minus_eta**3)

dg_deta=-1.0D0/(2.0d0*one_minus_eta**3)+3*(1.0d0-0.5d0*eta) &
     &	/one_minus_eta**4

dg_drho = eta / rho_local * dg_deta


!---if correction=0 then the integral is calculated with the Chapman approximation
!---if correction=1 then integral is corrected (Chang&Sandler)

if (correction.eq.0)then
!---integral approximated---------------------------------------------------------
integral = kappa * ghs
dint_drho = kappa * dg_drho
!---------------------------------------------------------------------------------

else if(correction.eq.1)then
!---integral calculated with the Cahng and Sandler method-------------------------
call cs_integral(eta,lambda,I,deriv_I)

integral=kappa * I
dint_drho = kappa * (eta/rho_local) * deriv_I

end if
!---------------------------------------------------------------------------------

F_AB = dexp(epsilon/TK) - 1.0d0

!---delta_AB and its density derivative---------------------
delta_AB = sigc**3.0d0 * F_AB * integral

ddelta_AB_drho = F_AB * sigc**3.0d0 * dint_drho

rho_delta_AB = rho_local * delta_AB

drho_delta_AB_drho = delta_AB + rho_local * ddelta_AB_drho



!---fraction of molecules not bonded at site A = XA--------

if(i_assoc_sites.eq.2)then
!---2(1,1) association model-----------------------------------
! 2 association sites, one acceptor, one donor, both with same volume

term_assoc1 = 1.0d0 + 4.0d0 * rho_delta_AB

term_assoc2 = 1.0d0 / (dsqrt(term_assoc1) * rho_delta_AB) - &
&(-1.0d0 + dsqrt(term_assoc1))/(2.0d0 * rho_delta_AB**2.0d0)

XA = (-1.0d0 + dsqrt(term_assoc1)) / (2.0d0 * rho_delta_AB)

dXA_drho = term_assoc2 * drho_delta_AB_drho

!---helmholtz free energy---------------------------------
a_assoc = 2.0d0 * (dlog(XA) - XA/2.0d0) + 1.0d0 

da_assoc_drho = 2.0d0 * (1.0d0/XA - 0.5d0) * dXA_drho

!---compressibility factor--------------------------------
z_assoc = rho_local * da_assoc_drho

!-----------------------------------------------------------

else if(i_assoc_sites.eq.3)then
! 3(1,2) model, three sites, one acceptor and two donor (one O- site and two h+ sites)
XA =  (-(1.0d0 - rho_delta_AB) + dsqrt((1.0d0 + rho_delta_AB)**2.0d0 + 4.0d0 * rho_delta_AB))/ &
&  (4 * rho_delta_AB)

dXA_drho = -((-1 + rho_delta_AB + Sqrt(4*rho_delta_AB + (1 + rho_delta_AB)**2))*drho_delta_AB_drho)/ &
     &   (4.*rho_delta_AB**2) + (drho_delta_AB_drho + &
     &     (4*drho_delta_AB_drho + 2*(1 + rho_delta_AB)*drho_delta_AB_drho)/ &
     &      (2.*Sqrt(4*rho_delta_AB + (1 + rho_delta_AB)**2)))/(4.*rho_delta_AB)

!---helmholtz free energy---------------------------------
a_assoc = 2.0d0 * (dlog(XA) - XA + 1.0d0) + dlog(2.0d0 * XA - 1.0d0)

da_assoc_drho = 2.0d0 * dXA_drho * (1.0d0 / XA - 1.0d0 + 1.0d0 / (2.0d0 * XA - 1.0d0))

!---compressibility factor--------------------------------
z_assoc = rho_local * da_assoc_drho

else if(i_assoc_sites.eq.4)then
!---4(2,2) association model, rigorous for water---------------
! 4 sites, 2 h bond donor, 2 h bond acceptor
term_assoc1 = 1.0d0 + 8.0d0 * rho_delta_AB

XA = (-1.0d0 + dsqrt(term_assoc1)) / (4.0d0 * rho_delta_AB)

term_assoc2 = rho_delta_AB/dsqrt(1 + 8.0d0 * rho_delta_AB) - &
&   4.0d0 * (-1.0d0 + dsqrt(1.0d0 + 8.0d0 * rho_delta_AB))

dXA_drho = drho_delta_AB_drho/(rho_delta_AB * dsqrt(1.0d0+8.0d0 * rho_delta_AB)) - &
&   ((-1.0d0 + dsqrt(1.0d0 + 8.0d0 * rho_delta_AB)) * drho_delta_AB_drho) / &
&   (4.0d0 * rho_delta_AB**2.0d0)

!---helmholtz free energy---------------------------------
a_assoc = 2.0d0 * (2.0d0 * dlog(XA) - XA + 1.0d0)

da_assoc_drho = 2.0d0 * dXA_drho * (2.0d0 /XA - 1.0d0)

!---compressibility factor--------------------------------
z_assoc = rho_local * da_assoc_drho

end if

 


return
end subroutine

			
			
		

