subroutine cs_integral(eta,lambda,integral,deriv_integral)
implicit none

!	Checking single & double precision
integer, parameter :: single = SELECTED_REAL_KIND(p=6, r=37)
integer, parameter :: double = SELECTED_REAL_KIND(p=13)

real (KIND=double) :: eta
real (KIND=double) :: f,yp,ym,zd,zs
real (KIND=double) :: a1,a2,a3
real (KIND=double) :: t1,t2,t3
real (KIND=double) :: A,B,C
real (KIND=double) :: lambda,gamma,h
real (KIND=double) :: integral,deriv_integral
real (KIND=double) :: df,dzd,dzs,dyp,dym,dA,dB,dC
real (KIND=double) :: da1,da2,da3,dt1,dt2,dt3

h=1.0d0/3.0d0
!---the derivative of the integral is calculated wrt eta------------------------------------
gamma = lambda - 1.0d0

f = 3.0d0 + 3.0d0 * eta - eta**2.0d0

yp = (2.0d0 * eta * f)**(1.0d0/3.0d0) * (dsqrt(2.0d0 * eta**4.0d0/f**2.0d0 + 1.0d0) &
        & + 1.0d0)**(1.0d0/3.0d0)

ym = (2.0d0 * eta * f)**(1.0d0/3.0d0) * (dsqrt(2.0d0 * eta**4.0d0/f**2.0d0 + 1.0d0) &
        & - 1.0d0)**(1.0d0/3.0d0)

zd = yp - ym

zs = yp + ym

A = (-2.0d0 * eta + zd)/(1.0d0-eta)

B = (-2.0d0 * eta - 0.5d0 * zd)/(1.0d0-eta)

C = (dsqrt(3.0d0) * zs)/(2.0d0 * (1.0d0 - eta))

a1 = (-2*eta*(1 - eta - 3*eta**2) + (1 - 3*eta - 4*eta**2)*zd + (1 + eta/2.)*zd**2)/ &
     & (3.*(1 - eta)**2*(2*eta**2 + zd**2))

a2 = (eta*(2 + 4*eta - 3*eta**2) - (1 - 3*eta - 4*eta**2)*zd + 2*(1 + eta/2.)*zd**2)/ &
     & (3.*(1 - eta)**2*(2*eta**2 + zd**2))

a3 = (eta*(2 - 5*eta**2)*zd + (1 - 3*eta - 4*eta**2)*(4*eta**2 + zd**2))/(dsqrt(3.0d0)* &
     & (1 - eta)**2*(2*eta**2 + zd**2)*zs)


t1 = (1.0d0 - A + dexp(A*gamma)*(-1 + A*lambda))/A**2

t2 = (-B + dexp(B*gamma)*lambda*(B*Cos(C*gamma) + C*Sin(C*gamma)))/(B**2 + C**2) - &
     & (-B**2 + C**2 + dexp(B*gamma)*((B**2 - C**2)*Cos(C*gamma) + 2*B*C*Sin(C*gamma))) &
	 & /(B**2 + C**2)**2

t3 = (C + dexp(B*gamma)*lambda*(-(C*Cos(C*gamma)) + B*Sin(C*gamma)))/(B**2 + C**2) - &
     & (2*B*C + dexp(B*gamma)*(-2*B*C*Cos(C*gamma) + (B**2 - C**2)*Sin(C*gamma)))/ &
	 & (B**2 + C**2)**2


integral=a1 * t1 + a2 * t2 + a3 * t3

!---compute the derivative of the integral wrt eta-------------------------------------
df = 3.0d0 - 2.0d0 * eta

dyp = 2**h*h*(1 + (1 + (2*eta**4)/f**2)**0.5)**h* &
     &   (eta*f)**(-1 + h)* &
     &   (f + eta*df) + &
     &  (0.5*2**h*h*(1 + (1 + (2*eta**4)/f**2)**0.5)** &
     &      (-1 + h)*(eta*f)**h* &
     &     ((8*eta**3)/f**2 - &
     &       (4*eta**4*df)/f**3))/ &
     &   (1 + (2*eta**4)/f**2)**0.5

if(eta.gt.2.0d-4)then
dym =  2**h*h*(-1 + (1 + (2*eta**4)/f**2)**0.5)**h* &
     &   (eta*f)**(-1 + h)* &
     &   (f + eta*df) + &
     &  (0.5*2**h*h*(-1 + &
     &        (1 + (2*eta**4)/f**2)**0.5)**(-1 + h)* &
     &     (eta*f)**h* &
     &     ((8*eta**3)/f**2 - &
     &       (4*eta**4*df)/f**3))/ &
     &   (1 + (2*eta**4)/f**2)**0.5
else
dym=0.0d0
end if

dzd = dyp - dym

dzs = dyp + dym

dA = (-2*eta + zd)/(1 - eta)**2 + &
     &  (-2 + dzd)/(1 - eta)

dB = (-2*eta - 0.5*zd)/(1 - eta)**2 + &
     &  (-2 - 0.5*dzd)/(1 - eta)

dC = (Sqrt(3.0d0)*zs)/(2.*(1 - eta)**2) + &
     &  (Sqrt(3.0d0)*dzs)/(2.*(1 - eta))


dt1 = -(2.0d0 * (1.0d0-A+dexp(gamma * A) * (lambda * A - 1.0d0)) * dA) / A**3.0d0 &
   &  + (-dA + dexp(gamma * A) * lambda * dA + dexp(gamma * A) * gamma * (lambda * A &
   &  - 1.0d0) * dA) / A**2.0d0

dt2 = -(((-B + dexp(gamma*B)*Lambda*(B*Cos(Gamma*C) + C*Sin(Gamma*C)))* &
     &       (2*B*dB + 2*C*dC))/(B**2 + C**2)**2) + &
     &  (2*(-B**2 + C**2 + dexp(Gamma*B)*((B**2 - C**2)*Cos(Gamma*C) + 2*B*C*Sin(Gamma*C)))* &
     &     (2*B*dB + 2*C*dC))/(B**2 + C**2)**3 + &
     &  (-dB + dexp(Gamma*B)*Gamma*Lambda*(B*Cos(Gamma*C) + C*Sin(Gamma*C))*dB + &
     &     dexp(Gamma*B)*Lambda*(Cos(Gamma*C)*dB + Gamma*C*Cos(Gamma*C)*dC + &
     &        Sin(Gamma*C)*dC - gamma*B*Sin(gamma*C)*dC))/(B**2 + C**2) - &
     &  (-2*B*dB + dexp(gamma*B)*gamma*((B**2 - C**2)*Cos(gamma*C) + 2*B*C*Sin(gamma*C))* &
     &      dB + 2*C*dC + &
     &     dexp(gamma*B)*(2*C*Sin(gamma*C)*dB + 2*gamma*B*C*Cos(gamma*C)*dC + &
     &        2*B*Sin(gamma*C)*dC - gamma*(B**2 - C**2)*Sin(gamma*C)*dC + &
     &        Cos(gamma*C)*(2*B*dB - 2*C*dC)))/(B**2 + C**2)**2

dt3 = -(((C + dexp(gamma*B)*lambda*(-(C*Cos(gamma*C)) + B*Sin(gamma*C)))* &
     &       (2*B*dB + 2*C*dC))/(B**2 + C**2)**2) + &
     &  (2*(2*B*C + dexp(gamma*B)*(-2*B*C*Cos(gamma*C) + (B**2 - C**2)*Sin(gamma*C)))* &
     &     (2*B*dB + 2*C*dC))/(B**2 + C**2)**3 + &
     &  (dexp(gamma*B)*gamma*lambda*(-(C*Cos(gamma*C)) + B*Sin(gamma*C))*dB + dC + &
     &     dexp(gamma*B)*lambda*(Sin(gamma*C)*dB - Cos(gamma*C)*dC + &
     &        gamma*B*Cos(gamma*C)*dC + gamma*C*Sin(gamma*C)*dC))/ &
     &   (B**2 + C**2) - (2*C*dB + &
     &     dexp(gamma*B)*gamma*(-2*B*C*Cos(gamma*C) + (B**2 - C**2)*Sin(gamma*C))*dB + &
     &     2*B*dC + dexp(gamma*B)* &
     &      (-2*C*Cos(gamma*C)*dB - 2*B*Cos(gamma*C)*dC + &
     &        gamma*(B**2 - C**2)*Cos(gamma*C)*dC + &
     &        2*gamma*B*C*Sin(gamma*C)*dC + &
     &        Sin(gamma*C)*(2*B*dB - 2*C*dC)))/(B**2 + C**2)**2

da1 =  (2*(-2*eta*(1 - eta - 3*eta**2) + &
     &       (1 - 3*eta - 4*eta**2)*zd + &
     &       (1 + eta/2.)*zd**2))/ &
     &   (3.*(1 - eta)**3*(2*eta**2 + zd**2)) - &
     &  ((-2*eta*(1 - eta - 3*eta**2) + &
     &       (1 - 3*eta - 4*eta**2)*zd + &
     &       (1 + eta/2.)*zd**2)* &
     &     (4*eta + 2*zd*dzd))/ &
     &   (3.*(1 - eta)**2*(2*eta**2 + zd**2)**2) + &
     &  (-2*(-1 - 6*eta)*eta - 2*(1 - eta - 3*eta**2) + &
     &     (-3 - 8*eta)*zd + zd**2/2. + &
     &     (1 - 3*eta - 4*eta**2)*dzd + &
     &     2*(1 + eta/2.)*zd*dzd)/ &
     &   (3.*(1 - eta)**2*(2*eta**2 + zd**2))

da2 =  (2*(eta*(2 + 4*eta - 3*eta**2) - &
     &       (1 - 3*eta - 4*eta**2)*zd + &
     &       2*(1 + eta/2.)*zd**2))/ &
     &   (3.*(1 - eta)**3*(2*eta**2 + zd**2)) - &
     &  ((eta*(2 + 4*eta - 3*eta**2) - &
     &       (1 - 3*eta - 4*eta**2)*zd + & 
     &       2*(1 + eta/2.)*zd**2)* &
     &     (4*eta + 2*zd*dzd))/ &
     &   (3.*(1 - eta)**2*(2*eta**2 + zd**2)**2) + &
     &  (2 + 4*eta + (4 - 6*eta)*eta - 3*eta**2 - &
     &     (-3 - 8*eta)*zd + zd**2 - &
     &     (1 - 3*eta - 4*eta**2)*dzd + &
     &     4*(1 + eta/2.)*zd*dzd)/ &
     &   (3.*(1 - eta)**2*(2*eta**2 + zd**2))

da3 = (2*(eta*(2 - 5*eta**2)*zd + &
     &       (1 - 3*eta - 4*eta**2)*(4*eta**2 + zd**2) &
     &       ))/ &
     &   (Sqrt(3.0d0)*(1 - eta)**3*(2*eta**2 + zd**2)* &
     &     zs) - ((eta*(2 - 5*eta**2)*zd + &
     &       (1 - 3*eta - 4*eta**2)*(4*eta**2 + zd**2) &
     &       )*(4*eta + 2*zd*dzd))/ &
     &   (Sqrt(3.0d0)*(1 - eta)**2*(2*eta**2 + zd**2)**2* &
     &     zs) + (-10*eta**2*zd + &
     &     (2 - 5*eta**2)*zd + &
     &     (-3 - 8*eta)*(4*eta**2 + zd**2) + &
     &     eta*(2 - 5*eta**2)*dzd + &
     &     (1 - 3*eta - 4*eta**2)* &
     &      (8*eta + 2*zd*dzd))/ &
     &   (Sqrt(3.0d0)*(1 - eta)**2*(2*eta**2 + zd**2)* &
     &     zs) - ((eta*(2 - 5*eta**2)*zd + &
     &       (1 - 3*eta - 4*eta**2)*(4*eta**2 + zd**2) &
     &       )*dzs)/ &
     &   (Sqrt(3.0d0)*(1 - eta)**2*(2*eta**2 + zd**2)* &
     &     zs**2)


!----derivative of the integral
deriv_integral = da1 * t1 + a1 * dt1 + da2 * t2 + a2 * dt2 + &
&   da3 * t3 + a3 * dt3


end

