!-----------------------------------Polar / Quadrupolar contribution (12/2/2002 - A.Ghosh)------------------------

	subroutine multipolar_int(TK, rhox, xd, xq, mu, Q , dhs,a_mp,z_mp)

	implicit none

	integer i, n, j, index
	double precision TK   ! Temperature in Kelvin
	double precision rhox ! number density of segments (per ang^3)
	double precision dhs  ! Temp. dependent diameter
	double precision Q	  ! quadrupole moment of component i in 
	double precision mu	  ! Dipole moment of component i in debye
	double precision J2(6,15)	 ! Coeffecients of 6th order polynomial used 
	double precision JT(6,4)	 ! to fit integrals.
	double precision I2(15)      ! The integral used in second order term
	double precision dI_drho(15)  ! d(I2())/d(rhox)
	double precision IT(4)       ! The integral used in third order term
	double precision dIT_drho(4) ! d(IT())/d(rhox)
	double precision xd		! Fraction of dipolar segments in a component
	double precision xq		! Fraction of quadripolar segments in a component
	double precision a_mp, z_mp, g_mp	! MultiPolar contribution to
										! A, Z and G
	double precision Kb, PI	! Boltzmann constant, Pi
	double precision rho
	double precision A2, A3A, A3B
C	 contribution to rho*d(A/NKT)/d(rho)
	double precision rdA2_dr, rdA3A_dr, rdA3B_dr 
	double precision mu_2, mu_4, mu_6, Q_2, Q_4, Q_6



	Kb=8.31441D0/6.022045D+23
	PI=3.141592654d0

	!	1 debye = 1e-18 esu.cm; 1debye = 3.162 * 1e-25 (j/m^3)^0.5
	!   unit of Q is 1e-26 esu.cm^2


	mu_2 = mu**2.d0 *1.d-49 /(TK*kb*dhs**3.d0*1.d-30)
	mu_4 = mu_2*mu_2
	mu_6 = mu_4*mu_2

	Q_2 = Q**2.0d0*1d-69/(dhs**5.0d0*kb*TK*1d-50) 
	Q_4 = Q_2*Q_2
	Q_6 = Q_4*Q_2


	J2(1,6)=4.1888d0
	J2(2,6)=2.8287d0
	J2(3,6)=0.8331d0
	J2(4,6)=0.0317d0
	J2(5,6)=0.0858d0
	J2(6,6)=-0.0846d0

	J2(1,8)=2.5133d0
	J2(2,8)=2.1795d0
	J2(3,8)=1.0423d0
	J2(4,8)=0.2596d0
	J2(5,8)=0.1097d0
	J2(6,8)=-0.0573d0

	J2(1,10)=1.7952d0
	J2(2,10)=1.7551d0
	J2(3,10)=1.0376d0
	J2(4,10)=0.3890d0
	J2(5,10)=0.1561d0
	J2(6,10)=-0.0082d0

	J2(1,11)=1.5708d0
	J2(2,11)=1.5957d0
	J2(3,11)=1.0079d0
	J2(4,11)=0.4237d0
	J2(5,11)=0.1740d0
	J2(6,11)=0.0215d0

	J2(1,13)=1.2566d0
	J2(2,13)=1.347d0
	J2(3,13)=0.9326d0
	J2(4,13)=0.4562d0
	J2(5,13)=0.1906d0
	J2(6,13)=0.0865d0

	J2(1,15)=1.0472d0
	J2(2,15)=1.1631d0
	J2(3,15)=0.8552d0
	J2(4,15)=0.4506d0
	J2(5,15)=0.1913d0
	J2(6,15)=0.1465d0


	!  JT: 1=TD, 2=DDQ, 3=DQQ, 4=TQ

	JT(1,1)=16.4493d0
	JT(2,1)=19.8096d0
	JT(3,1)=6.3321d0
	JT(4,1)=-0.0932d0
	JT(5,1)=-1.1741d0
	JT(6,1)=-0.8321d0

	JT(1,2)=139.4906d0
	JT(2,2)=241.9354d0
	JT(3,2)=163.9581d0
	JT(4,2)=57.0537d0
	JT(5,2)=13.2686d0
	JT(6,2)=-22.3827d0

	JT(1,3)=139.4906d0
	JT(2,3)=298.9225d0
	JT(3,3)=283.4908d0
	JT(4,3)=161.7222d0
	JT(5,3)=81.7788d0
	JT(6,3)=-35.8868d0

	JT(1,4)=532.9586d0
	JT(2,4)=1287.3491d0
	JT(3,4)=1447.9762d0
	JT(4,4)=1026.3847d0
	JT(5,4)=608.4235d0
	JT(6,4)=-20.5065d0

	!  IT: IT(1)=I_TD, IT(2)=I_DDQ, IT(3)=I_DQQ, IT(4)=I_TQ
	! I2 is required at n = 6,8,10,11,13,15




c----------------------------------------------------------------


	do i=1, 3

	if (i.eq.1) then
	j=6
	else if (i.eq.2) then
	j=10
	else if (i.eq.3) then
	j=15
	end if

	I2(j) = J2(1,j) + J2(2,j) * rhox + J2(3,j) * rhox**2.0d0 + 
     &	J2(4,j) * rhox**3.0d0 + J2(5,j) * rhox**4.0d0 + 
     &	J2(6,j) * rhox**5.0d0

	dI_drho(j) = J2(2,j) + 2.0d0 * J2(3,j) * rhox + 
     &	3.0d0 * J2(4,j) * rhox**2.0d0 + 4.0d0 * J2(5,j) * rhox**3.0d0 
     &	+ 5.0d0 * J2(6,j)*rhox**4.0d0

	end do

c----------------------------------------------------------------------------

	
	do j=1,4
	IT(j) = JT(1,j) + JT(2,j) * rhox + JT(3,j) * rhox**2.0d0 + 
     &	JT(4,j) * rhox**3.0d0 + JT(5,j) * rhox**4.0d0 + 
     &	JT(6,j) * rhox**5.0d0
	
	dIT_drho(j) = JT(2,j) + 2.0d0 * JT(3,j) * rhox + 
     &	3.0d0 * JT(4,j) * rhox**2.0d0 + 
     &	4.0d0 * JT(5,j) * rhox**3.0d0 + 5.0d0 * JT(6,j) * rhox**4.0d0
	enddo


c	A2 = -rhox * ( mu_4 * I2(6) / 6.0d0 + 
c     &	mu_2  * Q_2 * I2(8) / 2.0d0 + 
c     &	7.0d0 * Q_4 * I2(10) / 10.0d0 )
c
c	A3A = rhox * ( 2.0d0 * mu_4 * Q_2 * I2(11)/5.0d0+ 
c     &	12.0d0 * mu_2 * Q_4 * I2(13) / 35.0d0 + 
c     &	36.0d0 * Q_6 * I2(15) / 245.0d0 ) 
c
c	A3B = rhox**2.0d0 * ( mu_6 * IT(1) / 54.0d0 + 
c     &	mu_4 * Q_2 * IT(2) / 480.0d0 + 
c     &	mu_2 * Q_4 * IT(3) / 640.0d0 + 
c     &    Q_6  * IT(4) / 6400.0d0 )

!	Above expresions for A are exact; discarding the presence of dipole and quadrupole on 
!	the same molecule gives :

	A2 = -rhox * ( xd**2.0d0 * mu_4 * I2(6) / 6.0d0 + 
     &	7.0d0 * xq**2.0d0 * Q_4 * I2(10) / 10.0d0)


	A3A = rhox * ( xq**2.0d0 * 36.0d0 * Q_6 * I2(15) / 245.0d0 ) 

	A3B = rhox**2.0d0 * ( xd**3.0d0 * mu_6 * IT(1)/54.0d0 + 
     &	xq**3.0d0 * Q_6 * IT(4)/6400.0d0 )

	a_mp = A2**2.0d0/(A2 - A3A -A3B)

!	Z_mp = rhox*(d(A)/d(rhox))

	rdA2_dr = A2 - rhox**2.0d0 * 
     &	( xd**2.0d0 * mu_4 * dI_drho(6)/6.0d0  + 
     &	7.0d0 * xq**2.0d0 * Q_4 * dI_drho(10)/10.0d0 )

	rdA3A_dr = A3A + rhox**2.0d0 * 
     &	(xq**2.0d0 * 36.0d0*Q_6 * dI_drho(15)/245.0d0) 

	rdA3B_dr = 2.0d0 * A3B + rhox**3.0d0 * 
     & 	(xd**3.0d0 * mu_6 * dIT_drho(1)/54.0d0
     &	 + xq**3.0d0 * Q_6 * dIT_drho(4)/6400.0d0 )

	z_mp = (2.0d0*a_mp/A2) * rdA2_dr - (a_mp/A2)**2.0d0 * 
     &	(rdA2_dr - rdA3A_dr - rdA3B_dr)

	return

	end subroutine