subroutine pc_dispersion(m,z3,dz3_dT,d2z3_dT2,d2z3_drho_dT, &
     &	d3z3_drho_dT2,dz3_drho,u0,sigc,TK_local,amf,damfdz, &
     &       damfdz2,damfdz3,damfdT,damfdT2,d2amf_drho_dT,  &
     &       d3amf_drho_dT2)

implicit none

!	Checking single & double precision
integer, parameter :: single = SELECTED_REAL_KIND(p=6, r=37)
integer, parameter :: double = SELECTED_REAL_KIND(p=13)

real (KIND=double) :: m,u0,sigc,v00
real (KIND=double) :: z3,dz3_dT,d2z3_dT2,d2z3_drho_dT,d3z3_drho_dT2,dz3_drho
real (KIND=double) :: TK_local,rho_local
real (KIND=double) :: amf,damfdz,damfdz2,damfdz3
real (KIND=double) :: damfdT,damfdT2
real (KIND=double) :: d2amf_drho_dT,d3amf_drho_dT2
real (KIND=double) :: C1,C2,C3,C4,C5
real (KIND=double) :: dC1dT,d2C1dT2,dC2dT,d2C2dT2
real (KIND=double) :: apar(7),bpar(7)
real (KIND=double) :: order1,order2
real (KIND=double) :: I1,I2,edI1dz,edI2dz,edI1dz2,edI2dz2,edI1dz3,edI2dz3
real (KIND=double) :: edI1dz4,edI2dz4
real (KIND=double) :: dI1dT,dI2dT,d2I1dT2,d2I2dT2
real (KIND=double) :: edI1dzdT,edI2dzdT,edI1dzdT2,edI2dzdT2
real (KIND=double) :: term1,dterm1_dz,dterm1_dz2,dterm1_dz3,dterm1_dT,dterm1_dT2
real (KIND=double) :: term2,dterm2_dT
real (KIND=double) :: term3,dterm3_dT
real (KIND=double) :: term4,dterm4_dT

integer :: i,j
real (KIND=double),parameter :: pi=3.141592654d0
real (KIND=double),parameter :: avog = 0.602252d0




!---get the m-dependent PC-SAFT universal paramaters------------------------
call pcsaft_universal(m,apar,bpar)

!---get the derivatives of C1 wrt z3----------------------------------------
!---note that C1 is actually 1/C1 from the paper----------------------------
call C1_derivatives(m,z3,C1,C2,C3,C4,C5)

order1=(m**2.0)*u0/TK_local
order2=order1*u0/TK_local
rho_local=z3/(pi/6.0d0*m*(1.0d0-0.12*dexp(-3*u0/TK_local))**3.0d0)

I1    = 0.d0
I2    = 0.d0
edI1dz = 0.d0    !d(z*I1)/dz
edI2dz = 0.d0    !d(z*I2)/dz
edI1dz2=0.0d0    !d2(z*I1)/dz2
edI2dz2=0.0d0    !d2(z*I2)/dz2
edI1dz3=0.0d0    !d3(z*I1)/dz3
edI2dz3=0.0d0    !d3(z*I2)/dz3
edI1dz4 = 0.0d0  !d4(z*I1)/dz4
edI2dz4 = 0.0d0  !d4(z*I2)/dz4
dI1dT=0.0d0      !dI1/dT
dI2dT=0.0d0      !dI2/dT
d2I1dT2 = 0.0d0  !d2I1dT2
d2I2dT2 = 0.0d0  !d2I2dT2
edI1dzdT = 0.0d0 !d2(z*I1)/dzdT
edI2dzdT = 0.0d0 !d2(z*I2)/dzdT
edI1dzdT2 = 0.0d0 !d3(z*I1)/dzdT2
edI2dzdT2 = 0.0d0 !d3(z*I2)/dzdT2

do j=0,6

I1  = I1 + apar(j+1)*z3**DBLE(j)
I2  = I2 + bpar(j+1)*z3**DBLE(j)

!---------first derivatives wrt z3-----------------------------------------------------
edI1dz = edI1dz + apar(j+1)*DBLE(j+1)*z3**DBLE(j)
edI2dz = edI2dz + bpar(j+1)*DBLE(j+1)*z3**DBLE(j)

!---------second derivatives wrt z3----------------------------------------------------
edI1dz2=edI1dz2 + apar(j+1)*dble(j+1)*dble(j)*z3**dble(j-1)
edI2dz2=edI2dz2 + bpar(j+1)*dble(j+1)*dble(j)*z3**dble(j-1)

!---------third derivatives wrt z3-----------------------------------------------------
edI1dz3=edI1dz3+apar(j+1)*dble(j+1)*dble(j)*dble(j-1)*z3**dble(j-2)

edI2dz3=edI2dz3+bpar(j+1)*dble(j+1)*dble(j)*dble(j-1)*z3**dble(j-2)

!--------fourth derivatives wrt z3-----------------------------------------------------
edI1dz4 = edI1dz4 + apar(j+1) * dble(j+1)*dble(j)*dble(j-1)* dble(j-2) * z3**dble(j-3)

edI2dz4 = edI2dz4 + bpar(j+1) * dble(j+1)*dble(j)*dble(j-1)* dble(j-2) * z3**dble(j-3)

!-------first derivatives wrt T--------------------------------------------------------
dI1dT=dI1dT+apar(j+1)*dble(j)*dz3_dT*z3**dble(j-1)
dI2dT=dI2dT+bpar(j+1)*dble(j)*dz3_dT*z3**dble(j-1)

!------second derivative wrt T---------------------------------------------------------
d2I1dT2 = d2I1dT2 + apar(j+1) * dble(j) * (d2z3_dT2 * z3**dble(j-1) + dble(j-1) * &
&         dz3_dT**2.0d0 * z3**dble(j-2))

d2I2dT2 = d2I2dT2 + bpar(j+1) * dble(j) * (d2z3_dT2 * z3**dble(j-1) + dble(j-1) * &
&         dz3_dT**2.0d0 * z3**dble(j-2))

!-----second mixed derivative wrt z3 and T---------------------------------------------
edI1dzdT = edI1dzdT + apar(j+1) * dble(j+1) * dble(j) * dz3_dT * z3**dble(j-1)

edI2dzdT = edI2dzdT + bpar(j+1) * dble(j+1) * dble(j) * dz3_dT * z3**dble(j-1)

!-----third mixed derivative wrt z3 and T----------------------------------------------
edI1dzdT2 = edI1dzdT2 + apar(j+1) * dble(j+1) * dble(j) * (d2z3_dT2 * z3**dble(j-1) + &
&         dble(j-1) * dz3_dT**2.0d0 * z3**dble(j-2))

edI2dzdT2 = edI2dzdT2 + bpar(j+1) * dble(j+1) * dble(j) * (d2z3_dT2 * z3**dble(j-1) + &
&         dble(j-1) * dz3_dT**2.0d0 * z3**dble(j-2))

end do

dC1dT = dz3_dT * C2

d2C1dT2 = d2z3_dT2 * C2 + dz3_dT**2.0d0 * C3

dC2dT = dz3_dT * C3

d2C2dT2 = d2z3_dT2 * C3 + dz3_dT**2.0d0 * C4

!-------derivatives of term1 wrt z3-----------------------------------------------------
	
term1 = edI2dz/C1 + C2 * z3 * I2

dterm1_dz = edI2dz2/C1 + 2.0d0 * C2 * edI2dz + C3 * z3 * I2

dterm1_dz2 = edI2dz3/C1 + 3.0d0 * C2 * edI2dz2 + 3.0d0 * C3 * edI2dz + C4 * z3 * I2

dterm1_dz3 = edI2dz4/C1 + 4.0d0 * C2 * edI2dz3 + &
&	 6.0d0 * C3 * edI2dz2 + 4.0d0 * C4 * edI2dz + C5 * z3 * I2

!------derivatives of term1 wrt T------------------------------------------------------------
dterm1_dT = dC1dT * edI2dz + edI2dzdT/C1 + dC2dT * z3 * I2 + C2 * (dz3_dT * I2 + z3 * dI2dT)

dterm1_dT2 = d2C1dT2 * edI2dz + 2.0d0 * dC1dT * edI2dzdT + edI2dzdT2/C1 + d2C2dT2 * z3 * I2 &
&    + 2.0d0 * dC2dT * (dz3_dT * I2 + z3 * dI2dT) + C2 * (d2z3_dT2 * I2 + 2.0d0 * dz3_dT *  &
&    dI2dT + z3 * d2I2dT2)

!-----derivatives of term2 wrt T--------------------------------------------------------------
term2 = dI1dT - I1/TK_local

dterm2_dT = d2I1dT2 - dI1dT/TK_local + I1/TK_local**2.0d0

!----derivatives of term3 wrt T---------------------------------------------------------------
term3 = dC1dT * I2 + dI2dT/C1 - 2.0d0 * C1 * I2/TK_local

dterm3_dT = d2C1dT2 * I2 + 2.0d0 * dC1dT * dI2dT + d2I2dT2/C1 - 2.0d0 * dC1dT * I2/TK_local &
&    -2.0d0 * dI2dT/C1/TK_local + 2.0d0 * I2/C1/TK_local**2.0d0

!------derivatives of term4 wrt T-------------------------------------------------------------
term4 = edI1dzdT - edI1dz/TK_local

dterm4_dT = edI1dzdT2 - edI1dzdT/TK_local + edI1dz/TK_local**2.0d0


!-----amf and its derivatives wrt z3---------------------------------------------------------
amf=(-2.0d0*pi*rho_local*I1*order1 -pi*m*rho_local*C1**(-1.0d0)*I2*order2)

damfdz = -2.0d0 * pi * (rho_local/z3) * order1 * edI1dz -pi * (rho_local/z3) * m * order2 * term1

damfdz2 = -2.0d0 * pi * (rho_local/z3) * edI1dz2 * order1- pi * (rho_local/z3) * m * order2 * dterm1_dz

damfdz3 = -2.0d0 * pi * (rho_local/z3) * edI1dz3 * order1 -pi * rho_local/z3 * m * dterm1_dz2 * order2

!----derivatives of amf wrt T------------------------------------------------------------------

damfdT = -2.0d0 * pi * rho_local * term2 * order1 - pi * rho_local * m * term3 * order2

damfdT2 = -2.0d0 * pi * rho_local * (dterm2_dT - term2/TK_local) * order1 &
&      -pi * rho_local * m * (dterm3_dT - 2.0d0 * term3/TK_local) * order2

!----mixed derivatives wrt rho and T-----------------------------------------------------------
d2amf_drho_dT = -2.0d0 * pi * term4 * order1 - pi * m * (dterm1_dT - 2.0d0 * term1/TK_local) * order2

d3amf_drho_dT2 = -2.0d0 * pi * (dterm4_dT - term4/TK_local) * order1 - pi * m * (dterm1_dT2 - &
&      2.0d0 * dterm1_dT/TK_local + 2.0d0 * term1/TK_local**2.0d0) * order2 + 2.0d0 * pi * m * &
&      (dterm1_dT - 2.0d0 * term1/TK_local) * order2/TK_local




end subroutine


