c	Program to calculate the phase equilibrium of pure polar compounds
c	Based on u-expansion. The program fits pure component parameters
c     to vapor pressure and saturated liquid density data
c
c	PC SAFT dispersion term included - June 2001 by Auleen Ghosh
c--------------------------------------------------------------------------
c	Quadrupolar and dipolar term included - December 2002 by Auleen Ghosh
c	Reference Larsen, Rasaiah and Stell, Mol. Phys., 33 (1977) 987-1027
c	The dipolar integrals are estimated by a different method from that in
c	jog et al.(1999). Both approach should give the same answer for the dipolar term.
c
c	Dipolar [1]/Quadrupolar[2] flag - i_polar introduced
c----------------------------------------------------------------------------
c	This is a cleaned out version of the code. A code with commented
c	out partial jacobian implementation subroutines is in the SAFT_HS 
c	directory.
c
c	Note to self :
c	sigc, sigc2 is sigma (Temperature independent diameter)
c	sig = d (Temperature dependent diameter)
c
	Implicit double precision(a-h,o-z)
	double precision m,kappa
	dimension  fjac1(2,2),df(2,2)
	INTEGER    LDFJAC, MM, NN, num_assoc_sites
      PARAMETER  (NN=5)
	INTEGER    IPARAM(6)
	double precision fjac(50,4), fscale(50),fdif(50),rparam(7),
     &           parfit(nn), parfitf(nn), xscale(nn),f(2),xlb(nn),
     &		   xub(nn),ffjac(50,4),ffjacd(50,4)
	double precision xd, xq, mu, Q
      integer info(50,4)		
	common/temp/TK
	common/par/sigc,u0,m,e_ii,dmu,xp,v00
	common/par_polar/ i_polar
	common/par_assoc/kappa,epsilon
	common/output1/plcal1(500),rlcal1(500),rvcal1(500)
	common/output2/plcal2(500),rlcal2(500),rvcal2(500)
	common /tol1/ itmax
	common /tol2/ errrel 
	common/expdat1/tkdat1(500),rholdat1(500),pvdat1(500)
	common/expdat2/tkdat2(500),rholdat2(500),pvdat2(500)
	common/nexp/ndp1,ndp2
	common/guess/den(2)
	common/flag/iflag
	common/disp/idisp_flag
	common/assoc/iassoc_flag, num_assoc_site
	external funfit, DCHJAC
c
	!data xscale/4*1.0/, fscale/50*1.0d0/, xlb/5.0,180,0.0,0.1/
	!data  xub/15.0,400.0,10.0,1.0/
c
      open(5,FILE='input.TXT')
	write(6,*)'Enter 1 for PC dispersion and 0 for HR dispersion term'
	read(5,*) idisp_flag
	write(6,*)idisp_flag
	write(6,*)'Enter 1 for dipolar molecule and 2 for Quadrupolar'
	read(5,*) i_polar
	write(6,*)i_polar

	write(6,*)'Enter 1 for associatiing fluids, 0 otherwise'
	read(5,*)iassoc_flag
	write(6,*)iassoc_flag

	if (iassoc_flag.eq.1) then
		write(6,*)'Enter the number of association sites (1 to 4)'
		read (5,*) num_assoc_site
		write (6,*) num_assoc_site
	end if

! to fix a parameter, just replace parfit(i) with the variable name here, below
! and in funfit.for 
! IMPORTANT- make doubly sure that the parfit(i) indices match the variable you think
! you adjusting in funfit. The surest sign that they don't is a MATH error. 
	write(6,*)'Enter v00, u0'
	read(5,*)parfit(1),parfit(2)
	write(6,*)parfit(1),parfit(2)
      write(6,*)'enter the chain length m'
      read(5,*)parfit(3)
      write(6,*) parfit(3)

c	write(6,*)'Reduced Dipole/Quadrupole moment in debye and fraction of segments
c	1 bearing dipoles'

	write(6,*)' Dipole/Quadrupole moment in debye/debye.A, mol fr. 
	1  of polar spheres'
	read(5,*)dmu,xp
	write(6,*)dmu,xp

	write(6,*)'Kappa_AB,epsilon_AB and lambda for associating fluids'
	read(5,*)parfit(4),parfit(5)
	write(6,*)parfit(4),parfit(5)

	write(6,*)'Enter e_ii'
	read(5,*)e_ii
	write(6,*)e_ii 
      write(6,*)' Enter the molecular weight'
      read(5,*) xmolw
      write(6,*) xmolw
	write(6,*)'Tolerance for convergence of the nonlinear equations:'
	read(5,*)errrel
	write(6,*)errrel
	write(6,*)'Enter the maximum number of iterations
     1    for solving nonlinear equations'
      read(5,*)itmax  
	write(6,*)itmax
	write(6,*)'Enter number of data points for vapor pressure'
	read(5,*)ndp1
	write(6,*)ndp1
	write(6,*)'Enter number of data points for density'
	read(5,*)ndp2
	write(6,*)ndp2
	write(6,*) 'Enter the vapor pressure data as T(K)',
	1' P(KPa)'
	do k=1,ndp1
	read(5,*) tkdat1(k),pvdat1(k)
	write(6,*) tkdat1(k),pvdat1(k)
	enddo
	write(6,*) 'Enter the saturated liquid density data as T(K)',
	1' rho_l(mole/cc)'
	do i=1,ndp2
	read(5,*)tkdat2(i),rholdat2(i)
	write(6,*)tkdat2(i),rholdat2(i)
	enddo
	pause
c
	rg = 82.0535d0
	tau = 0.74048d0
	BAR=0.986923
	AVOG=0.602252d0
	pi = 3.141592654d0
	R1=8.3143 
! Currently set to adjust all 5 parameters
	u0=parfit(2)
	v00=parfit(1)
	m=parfit(3)
	kappa=parfit(4)
	epsilon=parfit(5)
c	xp=parfit(6)
c	
	sigc=(v00*6.0d0*0.74048d0/pi/AVOG)**(1.0D0/3.0D0)
	sigc3=sigc**3
	den(1) = den(1) * sigc3 * AVOG
	den(2) = den(2) * sigc3 * AVOG
	DEN1=DEN(1)
	DEN2=DEN(2)
	iparam(1)=0
	mm=ndp1+ndp2
	LDFJAC=mm
	nvar=2

	open(20,FILE='init_guess')

	call funfit(mm,nn,parfit,fdif)
	write(6,*)'For Initial guess:'
	write(6,*)'TK,rhol,rhov,Psat,Psat(exp),%Diff in pr.'
      DEN1=DEN(1)
	DEN2=DEN(2)
	do i=1,ndp1
	write(6,56)tkdat1(i),rlcal1(i),rvcal1(i),plcal1(i),pvdat1(i),
	1 (abs(plcal1(i)-pvdat1(i))/pvdat1(i))*100.0d0
	write(20,56)tkdat1(i),rlcal1(i),rvcal1(i),plcal1(i),pvdat1(i),
	1 (abs(plcal1(i)-pvdat1(i))/pvdat1(i))*100.0d0

      enddo 
      close(20)

	pause

	ibtype=0

	CALL DU4LSF (IPARAM, RPARAM)
	iparam(3)=500   ! Maximum number of iterations
	iparam(4)=2000	! Maximum number of function evaluations
c
	iersvr=4
	ipact=1
	isact=0
	iflag=0
	call ERSET (IERSVR, IPACT, ISACT)
c	do idunlsf=1,50
	CALL DUNLSF (funfit, mm, nn, parfit, XSCALE, FSCALE, IPARAM, 
     &       RPARAM, parfitf, fdif, fjac, ldfjac)
c	icode=iercd()
c	parfit = 0.5*parfit + 0.5*parfitf
c	write(6,*)'icode = ',icode
c	if (icode.eq.0) exit
c	end do

	write(6,*)'***** Optimization done*******'
	write(6,58)iparam(3),iparam(4),iparam(5)
 58	format('Maximum number of iterations:',I4,//,' Maximum number 
	1	of function evaluations',I4,//,' Maximum number of Jacobian 
	2	evaluations',I4)

! Currently set up to adjust all five parameters, XP is constant. 
	v00=parfitf(1)
	u0=parfitf(2) 
	m=parfitf(3)
	kappa=parfitf(4)
	epsilon=parfitf(5)
c	xp=parfitf(6)

	write(6,*)'Final fitted parameters m,v00,u0,kappa,epsilon,xp, lambda:',
	1	parfitf(3), parfitf(1), parfitf(2),parfitf(4),parfitf(5)
     &    ,xp

      iflag=1
	call funfit(mm,nn,parfitf,fdif)

	nvar=2
	open(10,FILE='output.txt')
	write(6,*)'Final fitted parameters m,v00,u0,kappa,epsilon,xp:',
	1	parfitf(3), parfitf(1), parfitf(2),parfitf(4),parfitf(5)
     &    ,xp
	aadp=0.0d0
	aadr=0.0d0
	write(6,*)'SET 1:'
	do i=1,ndp1
	write(6,56)tkdat1(i),rlcal1(i),rvcal1(i),plcal1(i),pvdat1(i),
	1 (abs(plcal1(i)-pvdat1(i))/pvdat1(i))*100.0d0
	aadp=aadp+(abs(plcal1(i)-pvdat1(i))/pvdat1(i))*100.0d0
	write(10,56)tkdat1(i),rlcal1(i),rvcal1(i),plcal1(i),pvdat1(i),
	1 (abs(plcal1(i)-pvdat1(i))/pvdat1(i))*100.0d0
	enddo
	write(6,*)'SET 2:'
	do j=1,ndp2
	write(6,56)tkdat2(j),rlcal2(j),rvcal2(j),rholdat2(j),
	1 (abs(rlcal2(j)-rholdat2(j))/rholdat2(j))*100.0d0
	write(10,56)tkdat2(j),rlcal2(j),rvcal2(j),rholdat2(j),
	1 (abs(rlcal2(j)-rholdat2(j))/rholdat2(j))*100.0d0
	aadr=aadr+(abs(rlcal2(j)-rholdat2(j))/rholdat2(j))*100.0d0
	enddo
 56   format(2x,F7.2,2x,F10.8,2x,F10.8,2x,F11.4,2x,F10.4,2x,F15.5)
c      
	aadp=aadp/ndp1
	aadr=aadr/ndp2
	write(6,24)aadp,aadr
 24   format('Average absolute deviation in vap. pr.(%):',F10.4 /
     1  'Average absolute deviation in liq. density(%):',F10.4)

	close(5)
	close(10)
	pause
	end