Subroutine KA_calc(KA_result, ncp, TK, rhox, m, x, u0, e_ii, sig)



!	This subroutine calculates the KA value used in polydisperse SAFT calculations

implicit none

!	Checking single & double precision
integer, parameter :: single = SELECTED_REAL_KIND(p=6, r=37)
integer, parameter :: double = SELECTED_REAL_KIND(p=13)

!	Declaring calling variables
integer, intent(in) :: ncp									! number of components
real (KIND=double), intent(in) :: TK, rhox
real (KIND=double), intent(in), dimension(ncp) :: x, m, e_ii
real (KIND=double), intent(in), dimension(ncp, ncp) :: sig, u0
real (KIND=double), intent(out) :: KA_result				

!	Declaring local variables
real (KIND=double), dimension(ncp, ncp) :: c, cs
real (KIND=double), dimension(ncp) :: dhs
real (KIND=double) :: zeta0, z3minus_c
real (KIND=double) :: zeta1, zeta2, zeta3, z3minus, z3minus_s
real (KIND=double), dimension(ncp, ncp) :: g
real (KIND=double), dimension(ncp) :: sitos1, sitos1s, sitos1c

integer :: i,j



!	Declare local parameter
real (KIND=double), parameter :: pi = 3.141592654d0
real (KIND=double) :: pi_over6 = pi / 6.0D0


!	Meat starts here
forall (i=1:ncp)
	dhs(i) = sig(i,i) * (1.0D0 - 0.12D0 * exp(-3.0D0*u0(i,i) / TK))
	sitos1(i) = dhs(i) / sig(1,1)

end forall

!	Calculating c(i,j)
forall (i=1:ncp, j=1:ncp)
	c(i,j) = sitos1(i) * sitos1(j) / (sitos1(i)+sitos1(j))
end forall

forall (i=1:ncp)
	sitos1s(i) = sitos1(i) * sitos1(i)
	sitos1c(i) = sitos1s(i) * sitos1(i)
end forall

!	Calculating zeta and derivative of zeta wrt temperature

!write(6,*) 'm', m
!write(6,*) 'x', x
!pause

do i = 1, ncp
	zeta0 = x(i) * m(i) + zeta0
	zeta1 = x(i) * m(i) * sitos1(i) + zeta1
	zeta2 = x(i) * m(i) * sitos1s(i) + zeta2
	zeta3 = x(i) * m(i) * sitos1c(i) + zeta3

end do

zeta0 = pi_over6 * rhox * zeta0
zeta1 = pi_over6 * rhox * zeta1
zeta2 = pi_over6 * rhox * zeta2
zeta3 = pi_over6 * rhox * zeta3


z3minus = 1.0d0 - zeta3
z3minus_s = z3minus * z3minus						! (z3minus)^2
z3minus_c = z3minus_s * z3minus						! (z3minus)^3 

forall (i=1:ncp, j=1:ncp)
	cs(i,j) = c(i,j) ** 2
end forall



forall (i=1:1, j=1:ncp)
	g(i,j) = 1.0d0 / z3minus + 3.0d0 * c(i,j) * zeta2 / z3minus_s + &
	&	2.0d0 * cs(i,j) * zeta2**2 / z3minus_c
end forall

KA_result = dlog(g(1,1))

END SUBROUTINE

