subroutine fcn_equil(guess,results,variables)		! dneqnf
!subroutine fcn_equil(variables, guess, results)	! for DNEQBF
!subroutine fcn_equil(variables, variables2, guess, results)	! for dunlsf



!	This subroutine takes the SAFT parameters and phase I composition given by 
!	plydsp_SAFT_driver and calculates the following values:
!			PressureI - PressureII
!			chemPotI(i) - chemPotII(i)
!	to be used by dneqnf.
!
!	Record of revisions:
!			Date		Programmer				Description of Change
!			====		==========				=====================
!			9/01/00 	P.D. Ting				inception
!



use essentials
implicit none
!	Declaring passing variables

integer, intent(in) :: variables
real (KIND=double), dimension(variables) :: guess
real(KIND=double), intent(out), dimension(variables) :: results

!	Declaring local variables
real (KIND=double) :: rhox1, rhox2
real (KIND=double), dimension(ncp) :: x_known, x_unknown, x_mass_dsp, y_mass_dsp, mw_dsp
real (KIND=double), dimension(ncp+nsc-1) :: x_known2, x_unknown2
real (KIND=double) :: temp, sum, sum2
integer :: temp2, i, j, a, b
real (KIND=double) :: KA, KB, KA_new, KB_new
real (KIND=double), dimension(nsc) :: x_poly_known, x_poly_unknown
real (KIND=double) :: x_unknown_p
real (KIND=double) :: PMn_unknown
real (KIND=double) :: KA_result1, KA_result2
real (KIND=double) :: KB_result1, KB_result2
real (KIND=double) :: AVOG = 0.602252d0 

!	Declaring SAFT parameters
real (KIND=double), dimension(ncp) :: m, e_ii, v00
integer, dimension(ncp) :: N_sites
real (KIND=double), dimension(ncp, ncp) :: sig, u0, kij, g, g2
real (KIND=double), dimension(ncp, ncp, max, max,2) :: fa	
real (KIND=double), dimension(ncp, ncp, max, max) :: kappa	
real (KIND=double), dimension(ncp, ncp, max, max, 1) :: fb	
real (KIND=double) :: Prsm, Ursm, Arsm, Grsm, Srsm, Hrsm, Pcalc1, Pcalc2, Gtotal 
real (KIND=double), dimension(ncp) :: chpt1, chpt2, fugacity1, fugacity2, chpt_res1, chpt_res2 

real (KIND=double), dimension(nsc + ncp -1) :: m2, e_ii2, v002
integer, dimension(nsc + ncp -1) :: N_sites2
real (KIND=double), dimension(nsc + ncp -1, nsc + ncp -1) :: sig2, u02, kij2
real (KIND=double), dimension(nsc + ncp -1) :: chpt1_2, chpt2_2, fugacity1_2, fugacity2_2
real (KIND=double), dimension(nsc + ncp -1) :: chpt_res1_2, chpt_res2_2
real (KIND=double) :: temp11, temp12, temp13, temp14


!	Here's the meat
!****************************************************************
!	Converting SAFT parameters to appropriate lengths
do i = 1, ncp
  temp = m_tmp(i)
  m(i) = temp
  temp = e_ii_tmp(i)
  e_ii(i) = temp
  temp = v00_tmp(i)
  v00(i) = temp
  temp2 = N_sites_tmp(i)
  N_sites(i) = temp2

  do j = 1, ncp
	temp = sig_tmp(i,j)
	sig(i,j) = temp
	temp = u0_tmp(i,j)
	u0(i,j) = temp
	temp = kij_tmp(i,j)
	kij(i,j) = temp
  end do
end do

do i = 1, ncp
  do a = 1, N_sites(i)
 	do j = 1, ncp
  	  do b = 1, N_sites(j)
		temp = fa_tmp(i,j,a,b,2)	
		fa(i,j,a,b,2) = temp
		temp = fa_tmp(i,j,a,b,1)
		fa(i,j,a,b,1) = temp
		temp = kappa_tmp(i,j,a,b)
		kappa(i,j,a,b) = temp
		temp = fb_tmp(i,j,a,b,1)
		fb(i,j,a,b,1) = temp
	  end do
	end do
  end do
end do

!write(11,*) 'in fcn', iscale1, iscale2, iscale3, iscale4
!	Conditioning the inputs to SAFT

!rhox1 = guess(1)/(10.0d0**iscale1)			! density of phase 1
!rhox2 = guess(2)/(10.0d0**iscale2)			! density of phase 2
!KA = guess(3)/(10.0d0**iscale3)
!KB = guess(4)/(10.0d0**iscale4)

rhox1 = guess(1)	! density of phase 1
rhox2 = guess(2)			! density of phase 2
KA = guess(3)
KB = guess(4)

write(11,*) 'KA, KB', KA, KB
if ((rhox1.le.0.0d0).or.(rhox2.le.0.0d0)) then
  do i = 1, variables
    results(i) = 100000.0
  end do
  RETURN
end if

if ((abs(rhox1-rhox2)/rhox1) .lt. 1.0d-9) then
  do i = 1, variables
    results(i) = 100000.0
  end do
  RETURN
end if

write(11,*) 'rhox1', rhox1
write(11,*) 'rhox2', rhox2
!	Calculating the mole fraction of the pseudocomponents


do i = 1, nsc
  x_poly_known(i) = x_tmp(1) * pd_mfrac_tmp(i)
end do


!	Calculate mole fraction of the pseudocomponents in the guess phase
x_unknown_p = 0.0d0
do i = 1, nsc
!  x_poly_unknown(i) = dexp(log(x_poly_known(i))+KA+KB*MW_pc(i))
  x_poly_unknown(i) = x_poly_known(i) * dexp(KA+KB*MW_pc(i))
  x_unknown_p = x_unknown_p + x_poly_unknown(i)
end do


!  Reshaping the input compositions to real compositions (i.e. pseudocomponents are now
!	real components for use in PTthermoprops.

sum = 0.0d0
do i = 1, ncp
  x_known(i) = x_tmp(i)
end do

do i = 1, ncp-npc-1
  x_unknown(i+npc) = guess(4+i)
  sum = sum + x_unknown(i+npc)
end do

x_unknown(npc) = x_unknown_p
sum = sum + x_unknown(npc)
x_unknown(ncp) = 1.0d0 - sum

do i = 1, ncp 
  if (x_unknown(i).lt.0.0d0) then
    do j = 1, variables
	  results(j) = 100000.0
	end do
	RETURN
  end if
  if (x_known(i).lt.0.0d0) then
	do j = 1, variables
	  results(j) = 100000.0
	end do
    RETURN
  end if
end do

!	Calculate the average properties of phases II.  
sum = 0.0d0
sum2 = 0.0d0
do i = 1, nsc
  sum = sum + x_poly_unknown(i) 
  sum2 = sum2 + x_poly_unknown(i) * MW_pc(i)
end do

PMn_unknown = sum2/sum

write(11,*) 'PMn_unknown', PMn_unknown
write(11,*) 'PMn_known', PMn_known

!	Calculate the pressure difference between the two phases.
!	For phase I

m(1) = alpha(1) * PMn_known
write(11,*) 'm known', m


call PTthermo_props2(ncp, max, status11, TK, rhox1, m, x_known, u0, e_ii, kij, N_sites, fa, &
						& fb, kappa, sig, D, v00, Prsm, Ursm, Arsm, Grsm, Srsm, Hrsm, chpt1, &
						& Pcalc1, fugacity1, chpt_res1, Gtotal, disp_flag, g) 
temp13 = g(1,1)
temp11 = (chpt_res1(1) - dlog(temp13))/PMn_known


!	For phase II
!write(11,*) 'm', m
!write(11,*) 'x', x_unknown
!write(11,*) 'rhox2', rhox2
!write(11,*) 'TK', TK
do i = 1, ncp
  temp = m_tmp(i)
  m(i) = temp
  temp = e_ii_tmp(i)
  e_ii(i) = temp
  temp = v00_tmp(i)
  v00(i) = temp
  temp2 = N_sites_tmp(i)
  N_sites(i) = temp2

  do j = 1, ncp
	temp = sig_tmp(i,j)
	sig(i,j) = temp
	temp = u0_tmp(i,j)
	u0(i,j) = temp
	temp = kij_tmp(i,j)
	kij(i,j) = temp
  end do
end do


m(1) = alpha(1) * PMn_unknown

write(11,*) 'm unknown', m
call PTthermo_props2(ncp, max, status11, TK, rhox2, m, x_unknown, u0, e_ii, kij, N_sites, fa, &
						& fb, kappa, sig, D, v00, Prsm, Ursm, Arsm, Grsm, Srsm, Hrsm, chpt2, &
						& Pcalc2, fugacity2, chpt_res2, Gtotal, disp_flag, g2) 

temp14 = g2(1,1)
temp12 = (chpt_res2(1) - dlog(temp14))/PMn_unknown

!	Calculating the output vector values
results(1) = (Pcalc1 - Pcalc2)/8314/TK

if ((pcalc1.ge.999).and.(pcalc2.ge.999)) results(1) = 100000



write(11,*) 'Pcalc1, Pcalc2', pcalc1, pcalc2
write(11,*) 'x_known', x_known
write(11,*) 'x_unknown', x_unknown
write(11,*) 'rhox1', rhox1/sig_tmp(1,1)**3 / AVOG
write(11,*) 'rhox2', rhox2/sig_tmp(1,1)**3/AVOG

do i = 1, ncp-npc
  results(i+1) = (chpt1(i+npc)-chpt2(i+npc))
end do
write(11,*) 'chpts', chpt1(2), chpt2(2)

!	Finding the Ka values.
!	for phase I
!m(1) = alpha(1) * PMn_known
!call KA_calc(KA_result1, ncp, TK, rhox1, m, x_known, u0, e_ii, sig)


!	for phase II

!m(1) = alpha(1) * PMn_unknown
!call KA_calc(KA_result2, ncp, TK, rhox2, m, x_unknown, u0, e_ii, sig) 

!KA_new = dlog(rhox1/rhox2) + KA_result1 - KA_result2


KA_new = dlog(rhox1/rhox2) + dlog(temp13) - dlog(temp14)


results(ncp-npc+2) = (KA - KA_new)



!	Calculate the Kb values
!	for phase I

!	Reshape the dimension of components so that all the pseudocomponents are now real 
!	components


do i = 1, nsc+ncp-1
  temp = m_tmp2(i)
  m2(i) = temp
  temp = e_ii_tmp2(i)
  e_ii2(i) = temp
  temp = v00_tmp2(i)
  v002(i) = temp
  temp2 = N_sites_tmp2(i)
  N_sites2(i) = temp2

  do j = 1, nsc+ncp-1
	temp = sig_tmp2(i,j)
	sig2(i,j) = temp
	temp = u0_tmp2(i,j)
	u02(i,j) = temp
	temp = kij_tmp2(i,j)
	kij2(i,j) = temp
  end do
end do


!	Shaping the x vectors
do i = 1, nsc
x_known2(i) = x_poly_known(i)
x_unknown2(i) = x_poly_unknown(i)
end do

do i = 2, ncp
x_known2(nsc+i-1) = x_known(i)
x_unknown2(nsc+i-1) = x_unknown(i)
end do


!	For the known phase, calculate the chemical potential and other info
temp2 = nsc+ncp-1
call PTthermo_props(temp2, max, status11, TK, rhox1, m2, x_known2, u02, e_ii2, kij2, N_sites2, fa, &
						& fb, kappa, sig2, D, v002, Prsm, Ursm, Arsm, Grsm, Srsm, Hrsm, chpt1_2, &
						& Pcalc1, fugacity1_2, chpt_res1_2, Gtotal, disp_flag) 



call PTthermo_props(temp2, max, status11, TK, rhox2, m2, x_unknown2, u02, e_ii2, kij2, N_sites2, fa, &
						& fb, kappa, sig2, D, v002, Prsm, Ursm, Arsm, Grsm, Srsm, Hrsm, chpt2_2, &
						& Pcalc2, fugacity2_2, chpt_res2_2, Gtotal, disp_flag) 

KB_result1 = ( chpt_res1_2(1) - chpt_res1_2(nsc-1) ) / (MW_pc(1) - MW_pc(nsc-1) )
KB_result2 = ( chpt_res2_2(1) - chpt_res2_2(nsc-1) ) / (MW_pc(1) - MW_pc(nsc-1) )

KB_new = KB_result1 - KB_result2





results(ncp-npc+3) = (KB - KB_new)

count = count + 1

!  iscale1 = -int(log10(abs(results(1))))
!  iscale2 = -int(log10(abs(results(2))))
!  iscale3 = -int(log10(abs(results(3))))+3
!  iscale4 = -int(log10(abs(results(4))))
!results(1) = results(1)*(10**iscale1)
!results(2) = results(2)*(10**iscale2)
!results(3) = results(3)*(10**iscale3)
!results(4) = results(4)*(10**iscale4)
write(6,*) 'ka', KA_new
write(6,*) 'kb', KB_new


temp = 0.0d0
do i = 1, ncp-npc+3
  write(6,*) 'results', i, results(i)
  temp = temp + results(i)**2
end do


write(11,*) 'final norm', temp
write(11,*) ' '
END SUBROUTINE



