Program main_Tx_driver_infdil


!	This is the driver for L-L or L-V 2 phase SAFT.  This code, does the following : 
!
!   Composition of phase 1 and temperature is specified. Component 1 is infinitely
!   dilute in phase 2 (hence this composition is also known {pure component2}). dneqnf
!   optimises densities of the 2 phases to match pressure and chemical potential of component2 
!   in both phases.

!----------------------------------------------------------------------------------------------

!main_TX_driver
!Tx_fcn_equi

!Used for general 2 component 2 phase cloud point calculation

!-------------------------------------------------------------------------------------------

!main_TX_driver_infdil
!Tx_fcn_equi_infdil

!Used for 2 component 2 phase cloud point calculation when component 1 is infinitely
!dilute in phase 2. (=> phase 2 can be considered to be pure component 2)

!------------------------------------------------------------------------------------------


!	Record of revisions:
!			Date		Programmer				Description of Change
!			====		==========				=====================
!			9/01/00 	P.D. Ting				inception
!			10/01/00    Auleen Ghosh			3 phase (s-l-v) calculation

use essentials

implicit none

!	Declaring local (thermo_props) variables

real (KIND=double) :: rhox
real (KIND=double), allocatable, dimension(:) :: m
real (KIND=double), allocatable, dimension(:,:) :: u0
real (KIND=double), allocatable, dimension(:) :: e_ii
real (KIND=double), allocatable, dimension(:,:,:,:) :: kappa
real (KIND=double), allocatable, dimension(:,:) :: sig
real (KIND=double), allocatable, dimension(:) :: v00
real (KIND=double), allocatable, dimension(:,:) :: kij
real (KIND=double), allocatable, dimension(:) :: x
real (KIND=double), allocatable, dimension(:,:,:,:,:) :: fa
real (KIND=double), allocatable, dimension(:,:,:,:,:) :: fb
real (KIND=double), allocatable, dimension(:) :: MW, x_mass
integer, allocatable, dimension(:) :: N_sites
real (KIND=double) :: Prsm, Arsm, Srsm, Ursm, Grsm, Hrsm, Pcalc, Gtotal
real (KIND=double), allocatable, dimension(:) :: chpt, fugacity, chpt_res

!	Declaring local parameters
real (KIND=double) :: AVOG = 0.602252d0 			! Nav / (10^8)^3
real (KIND=double) :: R = 83.1451					! cm3 bar/mol /k
real (KIND=double) :: pi = 3.141592654d0
real (KIND=double) :: tau = 0.7404804897
real (KIND=double) :: epsilon = 205.00*1e-4			! (cc/mol.K) molar thermal expansivity

!	Declaring local working variables
real (KIND=double) :: den1, den2		!	density guess for phase 1 and 2
real (KIND=double) :: q	                !   temperature coefficient of density

integer :: i, j, a, b, k, l, w
character*20 filein, fileout
integer :: ndp             				! number of data points 
integer :: polymer						! polymer flag (0) for no and (1) for yes
real (KIND=double) :: sum, sum1, sum2, sum3
real (KIND=double), allocatable, dimension(:,:) :: molefrac
real (KIND=double), allocatable, dimension(:) :: TK_all
real (KIND=double), allocatable, dimension(:) :: x_unknown, x_known, xw
real (KIND=double), allocatable, dimension(:) :: chpt1, chpt2
real (KIND=double) :: Pcalc1, Pcalc2 
real (KIND=double) :: TK_stored,fnorm_stored



!	Declaring variables used by ERSET
integer :: iersvr = 4
integer :: ipact = 0
integer :: isact = 0

!	Declaring variables used by DNEQNF
real (KIND=double) :: errrel, fnorm, pure_variable
integer :: itmax, variables
real (KIND=double), allocatable, dimension(:) :: xguess, xresult
real (KIND=double), dimension(1) :: pure_xresult

real (KIND=double) :: x1_phase1, x1_phase2, Pset

!	Declaring variables used by iercd
integer :: icode,iercd

external Tx_fcn_equil_infdil

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


!	Calling input_parameters to get SAFT parameters.  these will be temperature and pressure
!		independent parameters of each system.

call input_parameters		! get the Chen & Kreglewski coefficients

write(6,*) 'This is 2 phase (Liquid-Vapour) Tx SAFT for 2 components'
write(6,*) ' '
write(6,*) 'The association term is for 4 site model  - modify if other model wanted'
write(6,*) ' '
write(6,*) ' '

!write(6,*) 'Enter the input file name'
!read(5,*) filein
!write(6,*) 'Enter the output file name'
!read(5,*) fileout 

!open(unit=10, file=filein, status='old')
!open(unit=11, file='alkane_water.out', status='unknown')

!	Reading input parameters

!read(10,*) ncp					! number of real components
ncp=2

allocate(x_unknown(ncp),x_known(ncp), xw(ncp), chpt1(ncp), chpt2(ncp))

write(6,*)'Enter guess density (mol/cc) of phase 1 (known phase)'
read(5,*) den1					! density guess (mol/cm3) for phase1
write(6,*)'Enter guess density (mol/cc) of phase 2 (unknown phase)'
read(5,*) den2     			! density guess (mol/cm3) for phase2

!write(6,*)'Enter guess composition of component 1 in phase 2'
!read(5,*) x1_phase2  		    ! unknown phase mole fraction of componenet1

x1_phase2=1.0d-20
 
!read(10,*) disp_flag			! dispersion flag (0) for C&K and (1) for PCSAFT
disp_flag=1
write(6,*)'disp_flag = ',disp_flag

!read(10,*) status11				! association flag (0) for no association and (1) for association
status11=0
write(6,*)'assoc_flag = ',status11

!read(10,*) polymer				! polymer flag (0) for mole frac and (1) for mass frac
polymer=1
write(6,*)'polymer_flag = ',polymer

!read(10,*) Pset					!Pressure in Bars



!	Reading the SAFT parameters

! hydrocarbon(1) and water(2)

!do i = 1,ncp					
!read(10,*) MW_tmp(i)			
!read(10,*) v00_tmp(i), u0_tmp(i,i), m_tmp(i)	
!read(10,*) N_sites_tmp(i)		! number of bonding sites on each component
!read(10,*) e_ii_tmp(i)		
!end do


!do i=1, ncp
!  do j=1, ncp
!    read(10,*) kij_tmp(i,j)		! binary mixture parameter input
!  end do
!end do


!do j = 1, ncp
!  do i = 1, N_sites_tmp(j)
!    do k=1, ncp
!	  do l=1, N_sites_tmp(k)  
!		read(10,*) fa_tmp(j,k,i,l,2), kappa_tmp(j,k,i,l)	! association energy & volume	  
!	  end do
!	end do
!  end do
!end do


!read(10,*) variables  ! number of variables used in root finding i.e., P1=P2, chempot1=chempot2..
!read(10,*) errrel	  ! relative error to stop DNEQNF
!read(10,*) itmax	  ! max number of function evaluations
!read(10,*) ndp		  ! number of trial data points


variables=2
ndp=1
errrel = 1.0d-11
itmax = 5000000


allocate(molefrac(ndp, ncp), TK_all(ndp))

do i = 1, ndp

  write(6,*)'Enter temperature (Kelvin)'
  read(5,*) TK_all(i)		!  temperature of point i

  if (polymer.eq.0 ) then

	sum = 0.0d0
	do j = 1, ncp-1
	 write(6,*)'Enter mole fraction of component 1 in phase 1 (known phase)'
	 read(5,*) molefrac(i,j)	!  the composition of phase I for point i
		sum = sum + molefrac(i,j)
	 end do
	molefrac(i,ncp) = 1.0d0 - sum
  
  end if

end do

!close(unit=10)


!	Write out inputs
write(11,*) ' '

if (disp_flag.eq.0) write(6,*) 'SAFT code for cloud point calculations'
if (disp_flag.eq.1) write(6,*) 'PCSAFT code for cloud point calculations'
write(11,*) '**************************************'

write(6,*) 'number of components: ', ncp
write(6,*) 'polymer flag (0)=no, (1)=yes:  ', polymer
write(6,*) 'association flag:  ', status11 
write(6,*) 'dispersion flag:  ', disp_flag 
write(6,*) 'number of data points:  ', ndp 
!write(6,*) 'density guess for phase1 (mol/cm3)', den1
!write(6,*) 'density guess for phase2 (mol/cm3)', den2

write(6,*) ' '
write(6,*) 'max number of function evaluation:  ', itmax
write(6,*) 'relative errors allowed:  ', errrel
write(6,*) 'number of eqns solved:  ', variables
write(6,*) ' '


!	Calculating what needs to be calculated


forall (i=1:ncp)
	sig_tmp(i,i) = (v00_tmp(i)*6.0d0*tau/pi/AVOG)**(1.0d0/3.0d0)
end forall

forall (i=1:ncp, j=1:ncp, i .ne. j)
	sig_tmp(i,j) = 0.5d0 * (sig_tmp(i,i) + sig_tmp(j,j))
end forall

!	converting v00 from mL/mol to angstroms /molecule
forall (i=1:ncp)
	v00_tmp(i) = v00_tmp(i) / AVOG
end forall


max = 0
do i = 1, ncp
	if (max .lt. N_sites_tmp(i)) then
	 max = N_sites_tmp(i)
	end if
end do

allocate (m(ncp), u0(ncp,ncp), e_ii(ncp), kappa(ncp,ncp, max, max), sig(ncp, ncp), v00(ncp), &
			& kij(ncp, ncp), x(ncp), fa(ncp, ncp, max, max, 2), fb(ncp, ncp, max, max, 1), &
			& N_sites(ncp), chpt(ncp), xguess(variables), xresult(variables))


if (polymer.eq.1) then			! convert mass fraction to mole fraction
  do i = 1, ndp
    sum = 0.0d0


    do j = 1, ncp - 1
	   write(6,*)'Enter mass fraction of component 1 in phase 1 (known phase)'
	   read(5,*) molefrac(i,j)	!  the composition of phase I for point i
	   sum = sum + molefrac(i,j) / MW_tmp(j)
    end do
	
	molefrac(i,ncp) = 1.0d0 - molefrac(i,1)	! valid for 2 component system only
	sum = sum + molefrac(i,ncp) / MW_tmp(ncp)

    forall (j=1:ncp)
      molefrac(i,j) =	molefrac(i,j)/MW_tmp(j)/sum
    end forall
  end do
end if

!	Starts the real calculations
!***************************************************************************

do w = 1, ndp

  if (w.ge.2) then
  end if


  TK = TK_all(w)
  do i = 1, ncp
    x_tmp(i) = molefrac(w,i)
  end do




!sum = 0.0d0
!do i = 1, ncp
!	sum = x_tmp(i) * m_tmp(i) * sig_tmp(i,i)**3 + sum
!end do

!if (phase .eq. 2) then					! treat as vapor
!	rhox = Pset / TK / R * AVOG * sig_tmp(1,1)**3 /100.0d0
!	reducedrho = rhox*sum / sig_tmp(1,1)**3
!	if (reducedrho .gt. 0.99) then
!			rhox = 0.10d0 / sum * sig_tmp(1,1)**3
!	end if
!
!else
!	rhox = 0.90d0 / sum * sig_tmp(1,1)**3		! treat as liquid
!end if


!den1 = 5.33d-4
!den2 = 0.055083d0


den1 = den1 * sig_tmp(1,1)**3 * AVOG
den2 = den2 * sig_tmp(1,1)**3 * AVOG

!	assigning association parameters

  do j = 1, ncp
    do i = 1, N_sites_tmp(j)
	  do k = 1, ncp
	    do l = 1, N_sites_tmp(k)
		  fb_tmp(j,k,i,l,1) = fa_tmp(j,k,i,l,2)/TK/TK * exp(fa_tmp(j,k,i,l,2)/TK)
		  fa_tmp(j,k,i,l,1) = exp(fa_tmp(j,k,i,l,2)/TK) - 1.0d0 
		  fb(j,k,i,l,1) = fb_tmp(j,k,i,l,1)
!		  kappa_tmp(j,k,i,l) = kappa_tmp(j,k,i,l)
	    end do
	  end do
    end do
  end do

!	setting the vector that will be varied in the minimization routine. 

  do i = 1, ncp
    x_known(i) = x_tmp(i)
  end do


  xguess(1) = den1
  xguess(2) = den2
 

!	DNEQNF calculates the densities and compositions of the phases
	do i=1,500
		call ERSET(iersvr, ipact, isact)
!		do j=1,5
!			write(11,*)'xguess_enter = ',xguess(j)
!		enddo
		call DNEQNF(Tx_fcn_equil_infdil, errrel, variables, itmax, xguess, xresult, fnorm)
		icode=iercd()
		write(6,*)'icode = ',icode		

		if (icode.eq.0) then

		exit
		  do j=1,2
		    xguess(j)=xresult(j)
	      enddo
		else 
	      do j=1,2
		    xguess(j)=0.8*xguess(j)+0.2*xresult(j)
		  enddo
		endif 
	enddo
		

write(6,*)'den1 = ',xresult(1)
write(6,*)'den2 = ',xresult(2)

  
		x_unknown(1)=x1_phase2
		x_unknown(2)=1.0d0 - x_unknown(1)
	
		do i=1,2
		write(6,*)'y = ',x_unknown(i)
		enddo

	if (polymer.eq.1) then
			sum = 0.0d0
			do j = 1, ncp
			 sum = sum + x_known(j) * MW_tmp(j)
			end do

			do j=1,ncp
			 x_known(j)=x_known(j)*MW_tmp(j)/sum
			enddo
	end if


		call final_properties(x_known, x_unknown, xresult(1), xresult(2), chpt1, chpt2, Pcalc1, Pcalc2)

		write(6,*) '************************************'
		
		write(6,*) 'TK', TK
		write(6,*) 'density of fixed phase', xresult(1)/sig_tmp(1,1)**3 / AVOG
		write(6,*) 'density of varied phase', xresult(2)/sig_tmp(1,1)**3 / AVOG
		write(6,*) 'composition of known phase', x_known
		write(6,*) 'composition of unknown phase', x_unknown
		write(6,*) 'fnorm', fnorm
		write(6,*)  ' '     
		write(6,*) 'Pcalc1, Pcalc2', Pcalc1, Pcalc2
		write(6,*) 'chpt1', chpt1
		write(6,*) 'chpt2', chpt2
		



enddo

END PROGRAM

 