Program main_Tx_driver_mcp


!	This is the driver for L-L or L-V phase SAFT.  This code, does the following : 
!	Given a temperature and  composition of  phase 1 it will calculate the densities of both
!   phases and composition of phase 2. 
!   This code is for multicomponent systems




!	Record of revisions:
!			Date		Programmer				Description of Change
!			====		==========				=====================
!			9/01/00 	P.D. Ting				inception

use essentials

implicit none

!	Declaring local (thermo_props) variables

real (KIND=double) :: rhox
real (KIND=double), allocatable, dimension(:) :: m
real (KIND=double), allocatable, dimension(:,:) :: u0
real (KIND=double), allocatable, dimension(:) :: e_ii
real (KIND=double), allocatable, dimension(:,:,:,:) :: kappa
real (KIND=double), allocatable, dimension(:,:) :: sig
real (KIND=double), allocatable, dimension(:) :: v00
real (KIND=double), allocatable, dimension(:,:) :: kij
real (KIND=double), allocatable, dimension(:) :: x
real (KIND=double), allocatable, dimension(:,:,:,:,:) :: fa
real (KIND=double), allocatable, dimension(:,:,:,:,:) :: fb
real (KIND=double), allocatable, dimension(:) :: MW, x_mass
integer, allocatable, dimension(:) :: N_sites
real (KIND=double) :: Prsm, Arsm, Srsm, Ursm, Grsm, Hrsm, Pcalc, Gtotal
real (KIND=double), allocatable, dimension(:) :: chpt, fugacity, chpt_res

!	Declaring local parameters
real (KIND=double) :: AVOG = 0.602252d0 			! Nav / (10^8)^3
real (KIND=double) :: R = 83.1451					! cm3 bar/mol /k
real (KIND=double) :: pi = 3.141592654d0
real (KIND=double) :: tau = 0.7404804897
real (KIND=double) :: epsilon = 205.00*1e-4			! (cc/mol.K) molar thermal expansivity

!	Declaring local working variables
real (KIND=double) :: den1, den2		!	density guess for phase 1 and 2
real (KIND=double) :: q	                !   temperature coefficient of density

integer :: i, j, a, b, k, l, w, index
character*20 filein, fileout
integer :: ndp             				! number of data points 
integer :: polymer						! polymer flag (0) for no and (1) for yes
real (KIND=double) :: sum, sum1, sum2, sum3
real (KIND=double), allocatable, dimension(:,:) :: molefrac
real (KIND=double), allocatable, dimension(:) :: TK_all
real (KIND=double), allocatable, dimension(:) :: x_unknown, x_known, xw,w_unknown
real (KIND=double), allocatable, dimension(:) :: chpt1, chpt2
real (KIND=double) :: Pcalc1, Pcalc2 
real (KIND=double) :: TK_stored,fnorm_stored, Kb, Ka



!	Declaring variables used by ERSET
integer :: iersvr = 4
integer :: ipact = 0
integer :: isact = 0

!	Declaring variables used by DNEQNF
real (KIND=double) :: errrel, fnorm, pure_variable
integer :: itmax, variables
real (KIND=double), allocatable, dimension(:) :: xguess, xresult
real (KIND=double), dimension(1) :: pure_xresult

real (KIND=double) :: Pset

!	Declaring variables used by iercd
integer :: icode,iercd

!	declaring variables used for quadrature

integer :: N_q, IWEIGH, NFIX
real (kind=double) :: ALP, BETA
real (KIND=double), allocatable, dimension(:) :: QX, QW
real (KIND=double), allocatable, dimension(:) :: QXFIX


external Tx_fcn_equil_mcp

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


!	Calling input_parameters to get SAFT parameters.  these will be temperature and pressure
!		independent parameters of each system.

call input_parameters		! get the Chen & Kreglewski coefficients

write(6,*) 'This is 2 phase (LV/LL) Tx SAFT for multicomponent system'
write(6,*) ' '
write(6,*) 'The association term is for 4 site model  - modify if other model wanted'
write(6,*) ' '

!write(6,*) 'Enter the input file name'
!read(5,*) filein
!write(6,*) 'Enter the output file name'
!read(5,*) fileout 

!open(unit=10, file=filein, status='old')
!open(unit=11, file='alkane_water.out', status='unknown')

!	Reading input parameters

!read(5,*) ncp					! number of real components
ncp=2

write(6,*)'ncp = ',ncp

allocate(x_unknown(ncp),x_known(ncp), xw(ncp), chpt1(ncp), chpt2(ncp))

write(6,*)'Enter guess density (mol/cc) of phase 1 (known phase)'
!read(5,*) den1					! density guess (mol/cm3) for phase1
den1=  0.01323466 !0.00851054d0 !0.006d0 
write(6,*)den1

write(6,*)'Enter guess density (mol/cc) of phase 2 (unknown phase)'
!read(5,*) den2     			! density guess (mol/cm3) for phase2
den2= 0.00050954

write(6,*) den2
	
!323.15	0.0101	0.9899	0.0142307	0.9857693	11.51910323	0.01323466	0.00050954	1.97E-23


      
write(6,*)'Enter guess composition in phase 2'

sum=0.0d0
!do i=1,ncp-1
!read(5,*) x_unknown(i)  		    ! unknown phase mole/mass fraction of component1
x_unknown(1)=0.00842307 !0.015d0 !0.1d0 !0.17489083d0 
!x_unknown(2)=0.016205242d0	               

!500.95	0.1	0.00009559	1018.245451	0.01351162	0.01363262	3.00E-09


!write(6,*)x_unknown(i)
sum=sum+x_unknown(1) !+x_unknown(2)
!enddo

write(6,*)'Last component composition will be calculated for you'

x_unknown(ncp) = 1.0d0 - sum
write(6,*)'x_unknown(ncp) = ',x_unknown(ncp)
open(10,File='output')

!read(5,*) disp_flag			! dispersion flag (0) for C&K and (1) for PCSAFT
disp_flag=1
!write(6,*)'disp_flag = ',disp_flag

!read(5,*) status11				! association flag (0) for no association and (1) for association
status11=0
!write(6,*)'assoc_flag = ',status11

!read(5,*) polymer				! polymer flag (0) for mole frac and (1) for mass frac
polymer=0
!write(6,*)'polymer_flag = ',polymer

!read(6,*) Pset					!Pressure in Bars


if (polymer.eq.1) then	! convert phase 2 guess composition to mole fraction
 
   sum = 0.0d0
    do j = 1, ncp
	   sum = sum + x_unknown(j) / MW_tmp(j)
    end do

    forall (j=1:ncp)
      x_unknown(j) = x_unknown(j)/MW_tmp(j)/sum
    end forall
end if



!read(5,*) variables  ! number of variables used in root finding i.e., P1=P2, chempot1=chempot2..
!read(5,*) errrel	 ! relative error to stop DNEQNF
!read(5,*) itmax		 ! max number of function evaluations
!read(5,*) ndp		 ! number of trial data points

variables=3
errrel=1d-25
itmax=50000000
ndp=10000
allocate(molefrac(ndp, ncp), TK_all(ndp))

do i = 1, ndp


  if (polymer.eq.0 ) then

	sum = 0.0d0
	do j = 1, ncp-1
     !write(6,*)'Enter temperature (Kelvin),mole fraction of component 1 in phase 1 (known phase)'
	 !read(5,*)  TK_all(i), molefrac(i,j)	!  TK, the composition of phase I for point i
     TK_all(i)=323.15d0 !-dble(i)*0.01 !156.465d0 !- dble(i)*0.0001d0
	  molefrac(i,1)=0.0101d0 + 1d-4*dble(i) !0.50005d0 -1.0d-7*dble(i)
		sum = sum + molefrac(i,j)
	end do
	molefrac(i,ncp) = 1.0d0 - sum
  
  elseif (polymer.eq.1) then			! convert mass fraction to mole fraction

    sum  = 0.0d0
	sum1 = 0.0d0
    do j = 1, ncp - 1
!	   write(6,*)'Enter temperature (Kelvin), Enter mass fraction of component 1 in phase 1 (known phase)'
!	   read(5,*) TK_all(i), molefrac(i,j)	!  TK, the composition of phase I for point i
	   TK_all(i)=500.0d0 !- dble(i-1)*0.1d0
	   molefrac(i,1)=0.1d0 !- 1.0d-5*dble(i-1) 
	   !molefrac(i,2)=0.01801126d0 + 1.0d-5*dble(i-1)

	   sum = sum + molefrac(i,j) / MW_tmp(j)
	   sum1  =sum1 + molefrac(i,j)
    end do
	
	molefrac(i,ncp) = 1.0d0 - sum1
	sum = sum + molefrac(i,ncp) / MW_tmp(ncp)

    forall (j=1:ncp)
      molefrac(i,j) =	molefrac(i,j)/MW_tmp(j)/sum
    end forall

  end if

end do








!	Write out inputs

if (disp_flag.eq.0) write(6,*) 'SAFT code for cloud point calculations'
if (disp_flag.eq.1) write(6,*) 'PCSAFT code for cloud point calculations'
write(11,*) '**************************************'

write(6,*) 'number of components: ', ncp
write(6,*) 'polymer flag (0)=no, (1)=yes:  ', polymer
write(6,*) 'association flag:  ', status11 
write(6,*) 'dispersion flag:  ', disp_flag 
write(6,*) 'number of data points:  ', ndp 
!write(6,*) 'density guess for phase1 (mol/cm3)', den1
!write(6,*) 'density guess for phase2 (mol/cm3)', den2

write(6,*) ' '
write(6,*) 'max number of function evaluation:  ', itmax
write(6,*) 'relative errors allowed:  ', errrel
write(6,*) 'number of eqns solved:  ', variables
write(6,*) ' '


!	Calculating what needs to be calculated


forall (i=1:ncp)
	sig_tmp(i,i) = (v00_tmp(i)*6.0d0*tau/pi/AVOG)**(1.0d0/3.0d0)
end forall

forall (i=1:ncp, j=1:ncp, i .ne. j)
	sig_tmp(i,j) = 0.5d0 * (sig_tmp(i,i) + sig_tmp(j,j))
end forall

!	converting v00 from mL/mol to angstroms /molecule
forall (i=1:ncp)
	v00_tmp(i) = v00_tmp(i) / AVOG
end forall


max = 0
do i = 1, ncp
	if (max .lt. N_sites_tmp(i)) then
	 max = N_sites_tmp(i)
	end if
end do

allocate (m(ncp), u0(ncp,ncp), e_ii(ncp), kappa(ncp,ncp, max, max), sig(ncp, ncp), v00(ncp), &
			& kij(ncp, ncp), x(ncp), fa(ncp, ncp, max, max, 2), fb(ncp, ncp, max, max, 1), &
			& N_sites(ncp), chpt(ncp), xguess(variables), xresult(variables))

!	assigning association parameters

  do j = 1, ncp
    do i = 1, N_sites_tmp(j)
	  do k = 1, ncp
	    do l = 1, N_sites_tmp(k)
		  fb_tmp(j,k,i,l,1) = fa_tmp(j,k,i,l,2)/TK/TK * exp(fa_tmp(j,k,i,l,2)/TK)
		  fa_tmp(j,k,i,l,1) = exp(fa_tmp(j,k,i,l,2)/TK) - 1.0d0 
		  fb(j,k,i,l,1) = fb_tmp(j,k,i,l,1)
!		  kappa_tmp(j,k,i,l) = kappa_tmp(j,k,i,l)
	    end do
	  end do
    end do
  end do




write(6,*)'TK, x_1 phase1, x_1 phase2, Pcalc1(bar), Pcalc2(bar), rho phase1(gmole/cc), rho phase2 (gmole/cc), kij, fnorm'
	
!	Starts the real calculations
!***************************************************************************

do w = 1, ndp


  TK = TK_all(w)
  kij(1,2)=3.3d-5 * TK+0.025776
  kij(2,1)=3.3d-5 * TK+0.025776
  do i = 1, ncp
    x_tmp(i) = molefrac(w,i)
  end do





  den1 = den1 * sig_tmp(1,1)**3 * AVOG
  den2 = den2 * sig_tmp(1,1)**3 * AVOG



!	setting the vector that will be varied in the minimization routine. 

  do i = 1, ncp
    x_known(i) = x_tmp(i)
  end do


  xguess(1) = den1
  xguess(2) = den2
  do i=3,2+ncp-1
    j=i-2
    xguess(i) = x_unknown(j)
  enddo
 
call Tx_fcn_equil_mcp(xguess,xresult,variables)

!	DNEQNF calculates the densities and compositions of the phases
	do i=1,5
		call ERSET(iersvr, ipact, isact)
!		do j=1,5
!			write(11,*)'xguess_enter = ',xguess(j)
!		enddo
		call DNEQNF(Tx_fcn_equil_mcp, errrel, variables, itmax, xguess, xresult, fnorm)
		icode=iercd()
!		write(6,*)'icode = ',icode		

		if (icode.eq.0) then

		exit
		  do j=1,variables
		    xguess(j)=xresult(j)
	      enddo
		else 
	      do j=1,variables
		    xguess(j)=0.8*xguess(j)+0.2*xresult(j)
		  enddo
		endif 
	enddo

  sum=0.0d0
  do i=3,2+ncp-1
    j=i-2
    x_unknown(j)=xresult(i)
	sum=sum+x_unknown(j)
  enddo

  x_unknown(ncp)= 1.0d0 - sum
 
  

		call final_properties(x_known, x_unknown, xresult(1), xresult(2), chpt1, chpt2, Pcalc1, Pcalc2,Hrsm)

!		write(6,*) '************************************'

!------------------
! guess for next set 

den1 = xresult(1)/sig_tmp(1,1)**3 / AVOG
den2 = xresult(2)/sig_tmp(1,1)**3 / AVOG
 
!-----------------
		
!		write(6,*) 'TK', TK
!		write(6,*) 'density of fixed phase', den1
!		write(6,*) 'density of varied phase', den2

!write(6,*)'x_known(1) = ',x_known(1)
!write(6,*)'x_known(2) = ',x_known(2)
!write(6,*)'x_unknown(1) = ',x_unknown(1)
!write(6,*)'x_unknown(2) = ',x_unknown(2)




! If polymer system convert back to mass fraction

	if (polymer.eq.1) then
			sum = 0.0d0
			do j = 1, ncp
			 sum = sum + x_known(j) * MW_tmp(j)
			end do

			do j=1,ncp
			 x_known(j)=x_known(j)*MW_tmp(j)/sum
			enddo
	end if
	
	do index=1,ncp
		xw(index) = x_unknown(index)
	end do
	
	if (polymer.eq.1) then
			sum = 0.0d0
			do j = 1, ncp
			 sum = sum + xw(j) * MW_tmp(j)
			end do

			do j=1,ncp
			 xw(j)=xw(j)*MW_tmp(j)/sum
			enddo
	end if

!		write(6,*) 'composition of known phase', x_known
!		write(6,*) 'composition of unknown phase', x_unknown
!		write(6,*) 'fnorm', fnorm
!		write(6,*)  ' '     

		!write(6,*) 'Pcalc1, Pcalc2', Pcalc1, Pcalc2
		!write(6,*) 'chpt_1', chpt1(1), chpt1(2) 
		!write(6,*) 'chpt_2', chpt2(1), chpt2(2)


if ((w.eq.1).or.(mod(w,100).eq.0).or.(w.eq.ndp)) then
!4 format('', F8.5,2x,F8.5,2x,F12.4,2x,e12.4)
!3 format('',F8.2,2x,2(E12.4,2x),2(F12.4,2x),2x,2(e12.4,2x),F10.5,2x,E12.4)
!write(6,3)TK_all(w),x_known(1),xw(1),Pcalc1,Pcalc2, den1,den2,kij_tmp(1,2), fnorm
!write(6,4)x_known(1), xw(1), Pcalc1, fnorm

!3 format('',F8.2,2x,F12.8,2x,F12.6,2x,F12.6,2x,F12.6,2x,E12.4)
3 format('',f8.2,2x,2(F15.8,2x),2(F15.8,2x),2x,F15.8,2x,F15.8,2x,F15.8,2x,E12.4,2x,F15.8)
write(10,3)TK_all(w),x_known(1),x_known(2),xw(1),xw(2),Pcalc1,den1,den2, fnorm !,Hrsm
write(6,3)TK_all(w),x_known(1),x_known(2),xw(1),xw(2),Pcalc1,den1,den2, fnorm !,Hrsm

!3 format('',e12.4,2x,3(E12.4,2x),F12.4,2x,2(e12.4,2x),E12.4)
!write(6,3)x_known(2),xw(1),xw(2),xw(3),Pcalc1,den1,den2,fnorm

!3 format('',F10.5,2x,3(E12.4,2x),F12.4,2x,2(e12.4,2x),E12.4)
!write(6,3)kij_tmp(1,2),xw(1),xw(2),xw(3),Pcalc1,den1,den2,fnorm

!3 format('',F8.2,2x,3(E12.4,2x),2(F12.4,2x),2x,2(e12.4,2x),F10.5,2x,E12.4)
!write(6,3)TK_all(w),xw(1),xw(2),xw(3),Pcalc1,Pcalc2, den1,den2,kij_tmp(1,2), fnorm

!4 format('',F8.2,2x,3(E12.4,2x),F12.4,2x,4(e12.4,2x),2x,E12.4)
!write(6,4)TK_all(w),xw(1),xw(2),xw(3),Pcalc1, den1,den2, Ka, Kb, fnorm
!4 format('',E12.4,2x,3(E12.4,2x),F12.4,2x,4(e12.4,2x),2x,E12.4)
!write(6,4)x_known(1),xw(1),xw(2),xw(3),Pcalc1, den1,den2, Ka, Kb, fnorm

end if

enddo

close(10)

END PROGRAM

 