program main_driver

!	This is the main driver used to fit kij parameters in SAFT.  After the user enters the
!	various experimental T, P, and composition information, main_driver will call PTdriver4_3cmp
!	to fit a set of kij that best describe this experimetal data set.  At this point, this code
!	only works for a two component system.  Both Chen & Kreglewski and PC SAFT can be used
!	to optimize the kijs.

!	This code needs the following subroutines
!		PTdriver4_3cmps.f90 (and all its various associated subroutines)
!		exp_data.f90	(queueing the user for experimetnal data values)
!		exp_data_param
!		DUNLSF			(IMSL routine)

!	Record of revisions:
!			Date		Programmer				Description of Change
!			====		==========				=====================
!			11/12/99 	P.D. Ting				inception
!			8/07/00		"     "					added PC SAFT dispersion capability

use exp_data_param

implicit none

integer :: i, expswitch,N, ldfjac						! indices and counters

real(KIND=double), allocatable, dimension(:) :: expTK, exppsre, expmolefracL, expmolefracV
real(KIND=double) :: kij_guess12, kij_result12

!	IMSL routine usage
integer, dimension(6) :: iparam
real(KIND=double), dimension(7) :: rparam
real(KIND=double) :: xscale
real(KIND=double), allocatable, dimension(:) :: fscale, fvec
real(KIND=double), allocatable, dimension(:,:) :: fjac

!	IMSL DUVMIF
real(KIND=double) :: step, bound, xacc, kij_mini_uvmif, sum
integer :: maxfn

external kij_mini_uvmif
!	Meat starts here

!	Enter the experimental data
write(6,*) '*********  Enter the Experimental Data Set Below **********'
write(6,*) ' '
write(6,*) 'The number of system components is fixed at 2'

write(6,*) ' '
write(6,*) 'Enter the number of data points'
read(5,*) ndatapoint
write(6,*) ndatapoint
write(6,*) ' '
 




allocate (expTK(ndatapoint), exppsre(ndatapoint), expmolefracL(ndatapoint), &
			& expmolefracV(ndatapoint), fscale(ndatapoint*2), fvec(ndatapoint*2), &
			& fjac(ndatapoint*2, 1)) 

write(6,*) 'To enter the experimental data by hand, enter (1), else enter (2)'
read(5,*) expswitch
write(6,*) ' '

if (expswitch .eq. 1) then
	do i = 1, ndatapoint
		write(6,*) 'Enter the Pressure (bar)'
		read(5,*) exppsre(i)
		write(6,*) 'Enter the temperature (K)'
		read(5,*) expTK(i)
		write(6,*) 'Enter the LIQUID phase mole fraction'
		read(5,*) expmolefracL(i)
		write(6,*) 'Enter the VAPOR phase mole fraction'
		read(5,*) expmolefracV(i)
		write(6,*) ' '
	end do

else if (expswitch .eq. 2) then
	do i = 1, ndatapoint
		exppsre(i) = exppsre_tmp(i)
		expTK(i) = expTK_tmp(i)
		expmolefracL(i) = expmolefracL_tmp(i)
		expmolefracV(i) = expmolefracV_tmp(i)
	end do

end if

write(6,*) ' ' 
write(6,*) 'Guess a kij(1,2)'
read(5,*) kij_guess12
write(6,*) kij_guess12

!	Call DUNLSF to calculate the optimum k(1,2)
M=ndatapoint*2
N=1
xscale = 1.0
ldfjac = M
step = 0.001
bound = 0.2
xacc = 1.0d-6
maxfn = 500000

forall (i=1:M)
	fscale(i) = 1.0d0
end forall


call DU4LSF(iparam, rparam)
iparam(1) = 1
iparam(3) = 300
iparam(4) = 1200

!call DUNLSF(kij_mini, M, N, kij_guess12, xscale, fscale, iparam, rparam, kij_result12, fvec, &
!			& fjac, ldfjac) 

call DUVMIF(kij_mini_uvmif, kij_guess12, step, bound, xacc, maxfn, kij_result12)


write(6,*) 'hello hello'
write(6,*) ' ' 
write(6,*) 'final kij value', kij_result12
!write(6,*) 'final residuals', fvec
pause

sum = kij_mini_uvmif(kij_result12)

write(6,*) 'final residual', sum
end program 