subroutine SLV_fcn_equil(guess,results,variables)

!	This subroutine takes the SAFT parameters and phase I composition given by 
!	SLV_SAFT_driver and calculates the following values:
!			PressureI - PressureII
!			chemPotI(i) - chemPotII(i)
!	to be used by dneqnf.
!
!	Record of revisions:
!			Date		Programmer				Description of Change
!			====		==========				=====================
!			9/01/00 	P.D. Ting				inception
!						A.Ghosh					modified for s-l-v flash
!

 

use essentials

!	Declaring passing variables

integer, intent(in) :: variables
real (KIND=double), intent(in), dimension(variables) :: guess
real(KIND=double), intent(out), dimension(variables) :: results

!	Declaring local variables
real (KIND=double) :: rhox1, rhox2
real (KIND=double), dimension(ncp) :: x_known, x_unknown
real (KIND=double) :: temp, sum, sum2
! pure_poly_fug, fugacity0,chpt0, chpt_res0, Pcalc0
integer :: temp2, i, j, a, b

!	Declaring SAFT parameters
real (KIND=double), dimension(ncp) :: m, e_ii, v00
integer, dimension(ncp) :: N_sites
real (KIND=double), dimension(ncp, ncp) :: sig, u0, kij
real (KIND=double), dimension(ncp, ncp, max, max,2) :: fa	
real (KIND=double), dimension(ncp, ncp, max, max) :: kappa	
real (KIND=double), dimension(ncp, ncp, max, max, 1) :: fb	
real (KIND=double) :: Prsm, Ursm, Arsm, Grsm, Srsm, Hrsm, Pcalc1, Pcalc2, Gtotal 
real (KIND=double), dimension(ncp) :: chpt1, chpt2, fugacity1, fugacity2, chpt_res1, chpt_res2 

real (KIND=double) :: sitos1(ncp), dhs(ncp), sitos1c(ncp)
real (KIND=double) :: zeta3
real (KIND=double), parameter :: pi = 3.141592654d0
real (KIND=double) :: pi_over6 = pi / 6.0D0
real (KIND=double) :: poly_eq	!temporary dummy variable



!	Here's the meat
!****************************************************************
!	Converting SAFT parameters to appropriate lengths
do i = 1, ncp
  temp = m_tmp(i)
  m(i) = temp
  temp = e_ii_tmp(i)
  e_ii(i) = temp
  temp = v00_tmp(i)
  v00(i) = temp
  temp2 = N_sites_tmp(i)
  N_sites(i) = temp2

  do j = 1, ncp
	temp = sig_tmp(i,j)
	sig(i,j) = temp
	temp = u0_tmp(i,j)
	u0(i,j) = temp
	temp = kij_tmp(i,j)
	kij(i,j) = temp
  end do
end do

do i = 1, ncp
  do a = 1, N_sites(i)
 	do j = 1, ncp
  	  do b = 1, N_sites(j)
		temp = fa_tmp(i,j,a,b,2)	
		fa(i,j,a,b,2) = temp
		temp = fa_tmp(i,j,a,b,1)
		fa(i,j,a,b,1) = temp
		temp = kappa_tmp(i,j,a,b)
		kappa(i,j,a,b) = temp
		temp = fb_tmp(i,j,a,b,1)
		fb(i,j,a,b,1) = temp
	  end do
	end do
  end do
end do

open(unit=14, file='obs.out', status='old')

!	Conditioning the inputs to SAFT

rhox1 = guess(1)!/(10.0d0**iscale(1))			! density of phase 1
rhox2 = guess(2)!/(10.0d0**iscale(2))			! density of phase 2
x_known(1) = guess(5)!/(10.0d0**iscale(5))  	! Polymer in liquid 
x_known(2) = guess(4)!/(10.0d0**iscale(4))		! solvent in liquid phase
x_unknown(2) = guess(3)!/(10.0d0**iscale(3))	! solvent in vapour phase

  x_unknown(1)=0.0d0
  x_known(3) = 1.0d0-x_known(1)-x_known(2)
  x_unknown(3) = 1.0d0 - x_unknown(2)

!******************************************************** Rescaling variables
	
!	rhox1=rhox1/(10.0d0**iscale(1))
!	rhox2=rhox2/(10.0d0**iscale(2))
!	x_known(1)=x_known(1)/(10.0d0**iscale(6))
!	x_unknown(2)=x_unknown(2)/(10.0d0**iscale(3))
!	x_known(2)=x_known(2)/(10.0d0**iscale(4))


write(14,*)'this is rhox1',rhox1
write(14,*)'this is rhox2',rhox2
write(14,*)'this is x_unknown(2)',x_unknown(2)
write(14,*)'this is x_known(2)',x_known(2)
write(14,*)'this is x_known(1)',x_known(1)
write(14,*)' '

!********************************************************
if ((rhox1.le.0.0d0).or.(rhox2.le.0.0d0)) then
  do i = 1, variables
    results(i) = 100000
  end do
  RETURN
end if

!write(6,*) 'cr_internal = ',cr

if ((x_known(1).lt.0.0d0).or.(x_known(2).lt.0.0d0).or.&
	(x_unknown(2).lt.0.0d0)) then
  do i = 1, variables
    results(i) = 100000
  end do
  RETURN
end if


if ((x_known(1).gt.1.0d0).or.(x_known(2).gt.1.0d0).or.&
	(x_unknown(2).gt.1.0d0)) then
  do i = 1, variables
    results(i) = 100000
  end do
  RETURN
end if


if ((abs(rhox1-rhox2)/rhox1) .lt. 1.0d-9) then
  do i = 1, variables
    results(i) = 100000
  end do
  RETURN
end if

zeta3 = 0.0d0
forall (i=1:ncp)
	dhs(i) = sig(i,i) * (1.0D0 - 0.12D0 * exp(-3.0D0*u0(i,i) / TK))
	sitos1(i) = dhs(i) / sig(1,1)
end forall

forall (i=1:ncp)
	sitos1c(i) = sitos1(i) * sitos1(i) * sitos1(i)
end forall

!	Calculating zeta and derivative of zeta wrt temperature for phase I

do i = 1, ncp
	zeta3 = x_known(i) * m(i) * sitos1c(i) + zeta3
end do

zeta3 = pi_over6 * rhox1 * zeta3

if ( zeta3 .ge. 0.999d0) then
	do i = 1, variables
     results(i) = 100000
    end do
	return
end if

if ( zeta3 .lt. 0.0d0) then
	do i = 1, variables
     results(i) = 100000
    end do
	return
end if

!	Calculating zeta and derivative of zeta wrt temperature for phase II

do i = 1, ncp
	zeta3 = x_unknown(i) * m(i) * sitos1c(i) + zeta3
end do

zeta3 = pi_over6 * rhox2 * zeta3

if ( zeta3 .ge. 0.999d0) then
	do i = 1, variables
     results(i) = 100000
    end do
	return
end if

if ( zeta3 .lt. 0.0d0) then
	do i = 1, variables
     results(i) = 100000
    end do
	return
end if

!	Calculate the SAFT properties of the given phase (I)


call PTthermo_props(ncp, max, status11, TK, rhox1, m, x_known, u0, e_ii, kij, N_sites, fa, &
						& fb, kappa, sig, D, v00, Prsm, Ursm, Arsm, Grsm, Srsm, Hrsm, chpt1, &
						& Pcalc1, fugacity1, chpt_res1, Gtotal, disp_flag) 






!	Calculate the SAFT properties of the 2nd phase (II)

call PTthermo_props(ncp, max, status11, TK, rhox2, m, x_unknown, u0, e_ii,&
						& kij, N_sites, fa, &
						& fb, kappa, sig, D, v00, Prsm, Ursm, Arsm, Grsm, Srsm, Hrsm, chpt2, &
						& Pcalc2, fugacity2, chpt_res2, Gtotal, disp_flag) 


!	Calculating the output

!	if (TK.le.343) then
!	cr = 0.75
!	else
!	cr = (0.75/(1-2.7183))*(1-dexp((TM - TK)/(TM-343)))
!	endif

!	if (TK>TM) then
!	cr=0.0d0
!	endif

solid_fug=(-1)*((Hm_c/(8.314*Tm))*((Tm/TK)-1) + del_v*(0.1*Pcalc0)/(8.314*TK))*cr*un
! 0.1*Pcalc (conversion from bars to MPa)


results(1) = (fugacity1(1) + log(x_known(1)) - solid_fug - pure_poly_fug) 

!write(6,*)'const = ',solid_fug+pure_poly_fug
!write(6,*)'log(x_known(1)) = ',log(x_known(1))
!write(6,*)'fugacity1(1) = ',fugacity1(1)
write(6,*) 'results(1) = ',results(1)

do i = 2, ncp
  results(i) = abs(chpt1(i)-chpt2(i))
end do

results(4) = abs(Pcalc1 - Pcalc0)*100
results(5) = abs(Pcalc2 - Pcalc0)*100





END SUBROUTINE