subroutine fcn_equil(guess,results,variables)

!	This subroutine takes the SAFT parameters and phase I composition given by 
!	plydsp_SAFT_driver and calculates the following values:
!			PressureI - PressureII
!			chemPotI(i) - chemPotII(i)
!	to be used by dneqnf.
!
!	Record of revisions:
!			Date		Programmer				Description of Change
!			====		==========				=====================
!			9/01/00 	P.D. Ting				inception
!



use essentials

!	Declaring passing variables

integer, intent(in) :: variables
real (KIND=double), intent(in), dimension(variables) :: guess
real(KIND=double), intent(out), dimension(variables) :: results

!	Declaring local variables
real (KIND=double) :: rhox1, rhox2
real (KIND=double), dimension(ncp) :: x_known, x_unknown
real (KIND=double) :: temp, sum, sum2
integer :: temp2, i, j, a, b

!	Declaring SAFT parameters
real (KIND=double), dimension(ncp) :: m, e_ii, v00
integer, dimension(ncp) :: N_sites
real (KIND=double), dimension(ncp, ncp) :: sig, u0, kij
real (KIND=double), dimension(ncp, ncp, max, max,2) :: fa	
real (KIND=double), dimension(ncp, ncp, max, max) :: kappa	
real (KIND=double), dimension(ncp, ncp, max, max, 1) :: fb	
real (KIND=double) :: Prsm, Ursm, Arsm, Grsm, Srsm, Hrsm, Pcalc1, Pcalc2, Gtotal 
real (KIND=double), dimension(ncp) :: chpt1, chpt2, fugacity1, fugacity2, chpt_res1, chpt_res2 




!	Here's the meat
!****************************************************************
!	Converting SAFT parameters to appropriate lengths
do i = 1, ncp
  temp = m_tmp(i)
  m(i) = temp
  temp = e_ii_tmp(i)
  e_ii(i) = temp
  temp = v00_tmp(i)
  v00(i) = temp
  temp2 = N_sites_tmp(i)
  N_sites(i) = temp2

  do j = 1, ncp
	temp = sig_tmp(i,j)
	sig(i,j) = temp
	temp = u0_tmp(i,j)
	u0(i,j) = temp
	temp = kij_tmp(i,j)
	kij(i,j) = temp
  end do
end do

do i = 1, ncp
  do a = 1, N_sites(i)
 	do j = 1, ncp
  	  do b = 1, N_sites(j)
		temp = fa_tmp(i,j,a,b,2)	
		fa(i,j,a,b,2) = temp
		temp = fa_tmp(i,j,a,b,1)
		fa(i,j,a,b,1) = temp
		temp = kappa_tmp(i,j,a,b)
		kappa(i,j,a,b) = temp
		temp = fb_tmp(i,j,a,b,1)
		fb(i,j,a,b,1) = temp
	  end do
	end do
  end do
end do



!	Conditioning the inputs to SAFT

rhox1 = guess(1)			! density of phase 1
rhox2 = guess(2)			! density of phase 2


if ((rhox1.le.0.0d0).or.(rhox2.le.0.0d0)) then
  do i = 1, variables
    results(i) = 100000
  end do
  RETURN
end if

if ((abs(rhox1-rhox2)/rhox1) .lt. 1.0d-9) then
  do i = 1, variables
    results(i) = 100000
  end do
  RETURN
end if


sum = 0.0d0
sum2 = 0.0d0

do i = 1, ncp-1
  x_known(i) = x_tmp(i)
  x_unknown(i) = guess(i+2)
  sum = sum + x_known(i)
  sum2 = sum2 + x_unknown(i)
end do

x_known(ncp) = 1.0d0-sum
x_unknown(ncp) = 1.0d0 - sum2


do i = 1, ncp
  if (x_unknown(i).lt.0.0d0) then
    do j = 1, variables
	  results(j) = 100000
	end do
    RETURN
  end if
end do


!	Calculate the SAFT properties of the given phase (I)

call PTthermo_props(ncp, max, status11, TK, rhox1, m, x_known, u0, e_ii, kij, N_sites, fa, &
						& fb, kappa, sig, D, v00, Prsm, Ursm, Arsm, Grsm, Srsm, Hrsm, chpt1, &
						& Pcalc1, fugacity1, chpt_res1, Gtotal, disp_flag) 


!	Calculate the SAFT properties of the 2nd phase (II)


call PTthermo_props(ncp, max, status11, TK, rhox2, m, x_unknown, u0, e_ii, kij, N_sites, fa, &
						& fb, kappa, sig, D, v00, Prsm, Ursm, Arsm, Grsm, Srsm, Hrsm, chpt2, &
						& Pcalc2, fugacity2, chpt_res2, Gtotal, disp_flag) 

!	Calculating the output

do i = 1, ncp
  results(i+1) = chpt1(i)-chpt2(i)
end do

write(6,*) 'Pcalc1, pcalc2', pcalc1, pcalc2


results(1) = Pcalc1 - Pcalc2





END SUBROUTINE