subroutine test_polarmix(ncp, TK,rhox, zeta3, sig, m, x, dhs, dmu, & 
& xp, sitos1, Apolar, Zpolar,cmupolar, gpolar)

Implicit none
! Subroutine polarmix to calculate the dipolar contribution
! to thermodynamic properties. Does not use vdw 1 fluid theory for mixing.
! Programmed by Prasanna K. Jog
! on March 31, 2000

!	Checking single & double precision
integer, parameter :: single = SELECTED_REAL_KIND(p=6, r=37)
integer, parameter :: double = SELECTED_REAL_KIND(p=13)

integer, intent(in) :: ncp	! Number of components
integer :: i, j, k
!
real (KIND=double), intent(in) :: TK ! Temperature in Kelvin
real (KIND=double), intent(in) :: rhox						! number density of molecules (per ang^3)
real (KIND=double), intent(in) :: zeta3	! Packing fraction
real (KIND=double), intent(in), dimension(ncp, ncp) :: sig  ! Temperature independent segment diameter
real (KIND=double), intent(in), dimension(ncp) :: m			! chain length of molecule i
real (KIND=double), intent(in), dimension(ncp) :: x			! mole fraction of component i
real (KIND=double), intent(in), dimension(ncp) :: dhs		! temp. dependent diameter
real (KIND=double), intent(in), dimension(ncp) :: dmu		! Dipole moment of component i in debye
real (KIND=double), intent(in), dimension(ncp) :: xp		! Fraction of polar segments in a component
real (KIND=double), intent(in), dimension(ncp) :: sitos1	! dhs/sig(1,1)
!				   
real (KIND=double), intent(out) :: Apolar, Zpolar, gpolar	! Polar contribution to Helmholtz free energy, compressibility factor and Gibbs free energy
real (KIND=double), intent(out), dimension(ncp) :: cmupolar 
!
real (KIND=double) :: Kb, PI	! Boltzmann constant, Pi
real (KIND=double) :: j6		! The integral used in second order polar perturbation of free energy
real (KIND=double) :: kddd		! The integral used in third order polar perturbation of free energy
real (KIND=double) :: D, Afact, Bfact, Cfact, dj6dz, dj6dz2, dk_dz, dk_dz2, dd_sum
real (KIND=double) :: dd_su3, f2_dip, f3_dip, fdd2dz, fdd3dz, fdd_dz, myres, rho 
!
real (KIND=double), dimension(ncp) :: dd_my2	! Square of reduced dipole moment of component i : dmu**2/dhs**3*k*T
real (KIND=double), dimension(ncp) :: ddendx, dd_dx, dj6dx, dkdd_dx, dd3_dx
real (KIND=double), dimension(ncp) :: m2_ddq, m3_ddq, m_dd ! Intermediate variables
!
Kb=8.31441D0/6.022045D+23
PI=3.141592654d0
!
rho=rhox/(sig(1,1)**3)
D=zeta3/((PI/6.d0)*rho)
       do i = 1, ncp
!
         dd_my2(i)= dmu(i)**2.d0 *1.d-49 &
     &               /(Tk*kb*dhs(i)**3.d0*1.d-30)

!write(6,*)'dd_my2(i) = ',dd_my2(i)
!pause
         ddendx(i) = zeta3/D*m(i)*dhs(i)**3.d0
       end do
!
       Afact = (1.d0- 0.3618d0*6.d0/PI*zeta3 &
     &              - 0.3205d0*(6.d0/PI*zeta3)**2.d0 &
     &              + 0.1078d0*(6.d0/PI*zeta3)**3.d0 )
       Bfact = 1.d0 - 0.5236d0*6.d0/PI*zeta3
       Cfact = 1.d0 - 0.59056d0*6.d0/PI*zeta3 &
     &              + 0.20059d0*(6.d0/PI*zeta3)**2.d0
      j6= Afact/Bfact**2.d0
      dj6dz = (-0.3618d0*6.d0/PI &
     &                -0.3205d0*2.d0*(6.d0/PI)**2.d0*zeta3 &
     &                +0.1078d0*3.d0*(6.d0/PI)**3.d0*zeta3**2.d0 ) &
     &               / Bfact**2.d0 &
     &        + Afact*2.d0/Bfact**3.d0 *0.5236d0*6.d0/PI
      dj6dz2 = (      -0.3205d0*2.d0*(6.d0/PI)**2.d0 &
     &                +0.1078d0*6.d0*(6.d0/PI)**3.d0*zeta3 ) &
     &               / Bfact**2.d0 &
     &        +2.d0*(-0.3618d0*6.d0/PI &
     &                -0.3205d0*2.d0*(6.d0/PI)**2.d0*zeta3 &
     &                +0.1078d0*3.d0*(6.d0/PI)**3.d0*zeta3**2.d0 ) &
     &               /Bfact**3.d0 *2.d0*0.5236d0*6.d0/PI &
     &        + Afact*6.d0/Bfact**4.d0 *(0.5236d0*6.d0/PI)**2.d0
      kddd= (1.d0+0.62378d0*6.d0/PI*zeta3 &
     &                -0.11658d0*(6.d0/PI*zeta3)**2.d0 ) &
     &               / Cfact
      dk_dz = 1.d0/Cfact * (0.62378d0*6.d0/PI &
     &                -0.11658d0*2.d0*(6.d0/PI)**2.d0*zeta3 &
     &                -kddd*(- 0.59056d0*6.d0/PI &
     &                       +0.20059d0*2.d0*(6.d0/PI)**2.d0*zeta3))
      dk_dz=zeta3*dk_dz+kddd
      dk_dz2= 1.d0/Cfact * (-0.11658d0*2.d0*(6.d0/PI)**2.d0 &
     &                      -0.20059d0*2.d0*(6.d0/PI)**2.d0*kddd &
     &                      -2.d0*dk_dz*(- 0.59056d0*6.d0/PI &
     &                       +0.20059d0*2.d0*(6.d0/PI)**2.d0*zeta3))

      dd_sum = 0.d0
      dd_su3 = 0.d0
      DO i=1,ncp
        DO j=1,ncp
          dd_sum = dd_sum+ x(i)*x(j)*m(i)*m(j) &
     &             *dhs(i)**3.d0*dhs(j)**3.d0  &
     &             /(0.5d0*(dhs(i)+dhs(j)))**3.d0 &
     &             *xp(i)*xp(j)*dd_my2(i)*dd_my2(j)
          DO k=1,ncp
            dd_su3=dd_su3+x(i)*x(j)*x(k) &
     &         *m(i)*m(j)*m(k) &
     &         *dhs(i)**3.d0*dhs(j)**3.d0*dhs(k)**3.d0 &
     &         /(0.5d0*(dhs(i)+dhs(j))) &
     &         /(0.5d0*(dhs(j)+dhs(k))) &
     &         /(0.5d0*(dhs(i)+dhs(k))) &
     &         *xp(i)*xp(j)*xp(k) &
     &         *dd_my2(i)*dd_my2(j)*dd_my2(k)
          END DO
        END DO
      END DO

      f2_dip = -2.d0/9.d0*PI*rho*dd_sum*j6
!
      f3_dip= 5.d0/162.d0*PI**2.d0 *rho**2.d0 * dd_su3*kddd
      fdd2dz = -2.d0/9.d0*PI*6.d0/(PI*D)*dd_sum*(dj6dz*zeta3+j6)
      fdd3dz = 5.d0/162.d0*PI**2.d0 *(6.d0/(PI*D))**2.d0 *dd_su3 &
     &           *(dk_dz*zeta3*zeta3 + 2.d0*zeta3*kddd)
      IF ((DABS(f2_dip).GT.1.d-100).AND.(DABS(f3_dip).GT.1.d-100)) THEN
        Apolar= f2_dip*( 1.d0 / (1.d0-f3_dip/f2_dip) )
        fdd_dz = Apolar &
     &            * (  fdd2dz / f2_dip &
     &                  +  (fdd3dz-f3_dip/f2_dip*fdd2dz) &
     &                       / (f2_dip - f3_dip))
        zpolar = zeta3**2.d0  *6.d0/ PI /D * fdd_dz/rho
      ENDIF
!
      DO k=1,ncp
      
	  
	  dj6dx(k) = ((-0.3618d0*6.d0/PI &
     &                -0.3205d0*2.d0*(6.d0/PI)**2.d0*zeta3 &
     &                +0.1078d0*3.d0*(6.d0/PI)**3.d0*zeta3**2.d0) &
     &               / Bfact**2.d0 &
     &        + Afact*2.d0/Bfact**3.d0 *0.5236d0*6.d0/PI )*ddendx(k)

      dkdd_dx(k) = (1.d0/Cfact * (0.62378d0*6.d0/PI &
     &                -0.11658d0*2.d0*(6.d0/PI)**2.d0*zeta3 &
     &                -kddd*(- 0.59056d0*6.d0/PI &
     &                       +0.20059d0*2.d0*(6.d0/PI)**2.d0*zeta3)) &
     &              )*ddendx(k)
        dd_dx(k)  = 0.d0
        dd3_dx(k) = 0.d0
        DO i = 1,ncp
          dd_dx(k)  = dd_dx(k) + 2.d0*x(i)*m(i)*m(k) &
     &               *dhs(i)**3.d0*dhs(k)**3.d0 &
     &               /(0.5d0*(dhs(i)+dhs(k)))**3.d0 &
     &               *xp(i)*xp(k)*dd_my2(i)*dd_my2(k)
          DO j=1,ncp
            dd3_dx(k) = dd3_dx(k) &
     &         + 3.d0* x(i)*x(j) &
     &         *m(i)*m(j)*m(k) &
     &         *dhs(i)**3.d0*dhs(j)**3.d0*dhs(k)**3.d0 &
     &         /(0.5d0*(dhs(i)+dhs(j))) &
     &         /(0.5d0*(dhs(j)+dhs(k))) &
     &         /(0.5d0*(dhs(i)+dhs(k))) &
     &         *xp(i)*xp(j)*xp(k) &
     &         *dd_my2(i)*dd_my2(j)*dd_my2(k)
          END DO
        END DO

       m2_ddq(k)= -2.d0/9.d0*PI*rho* (dd_dx(k)*j6 + dd_sum*dj6dx(k))
       m3_ddq(k)= 5.d0/162.d0*PI**2.d0 *rho**2.d0 &
     &               * (dd3_dx(k)*kddd + dd_su3*dkdd_dx(k))

       m_dd(k)= Apolar &
     &          * (  m2_ddq(k) / f2_dip &
     &                +  (m3_ddq(k)-f3_dip/f2_dip*m2_ddq(k)) &
     &                     / (f2_dip - f3_dip))
       END DO

	myres=0.d0
	do i=1, ncp
		myres=myres-x(i)* m_dd(i)
	enddo
	do i=1,ncp
        cmupolar(i)=myres+apolar+zpolar +m_dd(i)
	enddo
!
    gpolar=apolar+zpolar

       return
       end