subroutine test_polarmix2(ncp, TK,rhox, zeta3, sig, m, x, dhs, dmu, & 
& xp, sitos1, Apolar, Zpolar,cmupolar, gpolar)

Implicit none
! Subroutine polarmix2 to calculate the dipolar contribution
! to thermodynamic properties.This uses the vander waal's one
! fluid mixing rule. Programmed by Auleen Ghosh
! on January 29, 2001

!	Checking single & double precision
integer, parameter :: single = SELECTED_REAL_KIND(p=6, r=37)
integer, parameter :: double = SELECTED_REAL_KIND(p=13)

integer, intent(in) :: ncp	! Number of components
integer :: i, j, k
!
real (KIND=double), intent(in) :: TK ! Temperature in Kelvin
real (KIND=double), intent(in) :: rhox						! number density of molecules (per ang^3)
real (KIND=double), intent(in) :: zeta3	! Packing fraction
real (KIND=double), intent(in), dimension(ncp, ncp) :: sig  ! Temperature independent segment diameter
real (KIND=double), intent(in), dimension(ncp) :: m			! chain length of molecule i
real (KIND=double), intent(in), dimension(ncp) :: x			! mole fraction of component i
real (KIND=double), intent(in), dimension(ncp) :: dhs		! temp. dependent diameter
real (KIND=double), intent(in), dimension(ncp) :: dmu		! Dipole moment of component i in debye
real (KIND=double), intent(in), dimension(ncp) :: xp		! Fraction of polar segments in a component
real (KIND=double), intent(in), dimension(ncp) :: sitos1	! dhs/sig(1,1)
!				   
real (KIND=double), intent(out) :: Apolar, Zpolar, gpolar	! Polar contribution to Helmholtz free energy, compressibility factor and Gibbs free energy
real (KIND=double), intent(out), dimension(ncp) :: cmupolar 
!
real (KIND=double) :: Kb, PI	! Boltzmann constant, Pi
real (KIND=double) :: j6		! The integral used in second order polar perturbation of free energy
real (KIND=double) :: kddd		! The integral used in third order polar perturbation of free energy
real (KIND=double) :: D, Afact, Bfact, Cfact, dj6dz, dj6dz2, dk_dz, dk_dz2, dd_sum
real (KIND=double) :: dd_su3, f2_dip, f3_dip, fdd2dz, fdd3dz, fdd_dz, myres, rho, rhostar_mix 
real (KIND=double) :: sig3_mix, sig3_mix_num, sig3_mix_den 
!
real (KIND=double), dimension(ncp) :: dd_my2	! Square of reduced dipole moment of component i : dmu**2/dhs**3*k*T
real (KIND=double), dimension(ncp) :: dd_dx, dj6dx, dkdd_dx, dd3_dx
real (KIND=double), dimension(ncp) :: dsig3_mix_num_dx, dsig3_mix_den_dx, dsig3_mix_dx
real (KIND=double), dimension(ncp) :: m2_ddq, m3_ddq, m_dd ! Intermediate variables
!
Kb=8.31441D0/6.022045D+23
PI=3.141592654d0
!
rho=rhox/(sig(1,1)**3)
D=zeta3/((PI/6.d0)*rho)

!calculate (mu*)^2=mu^2/kTsig^3
       do i = 1, ncp
         dd_my2(i)= dmu(i)**2.d0 *1.d-49 &
     &               /(Tk*kb*dhs(i)**3.d0*1.d-30)
       end do
!-------------------------------------

! sig^3 using van der waal's one fluid theory
	do i=1,ncp
		do j=1,ncp
		sig3_mix_num = sig3_mix_num + x(i)*x(j)*m(i)*m(j)*(0.5*(dhs(i)+dhs(j)))**3.d0
		sig3_mix_den = sig3_mix_den + x(i)*x(j)*m(i)*m(j)
		end do
	end do

	sig3_mix = sig3_mix_num/sig3_mix_den
	rhostar_mix = rho*sig3_mix

	do k=1,ncp
	  do j=1,ncp
	  dsig3_mix_num_dx(k)= dsig3_mix_num_dx(k)+2*m(k)*x(j)*m(j)*(0.5*(dhs(k)+dhs(j)))**3
	  dsig3_mix_den_dx(k)= dsig3_mix_den_dx(k)+2*m(k)*x(j)*m(j)
	  end do

	  dsig3_mix_dx(k)= (sig3_mix_den*dsig3_mix_num_dx(k) &
					 -sig3_mix_num*dsig3_mix_den_dx(k))/sig3_mix_den**2


	end do
!--------------------------------------

! j6 is the integral I2

       Afact = (1.d0- 0.3618d0*rhostar_mix &
     &              - 0.3205d0*(rhostar_mix)**2.d0 &
     &              + 0.1078d0*(rhostar_mix)**3.d0 )
       Bfact = 1.d0 - 0.5236d0*rhostar_mix
       Cfact = 1.d0 - 0.59056d0*rhostar_mix &
     &              + 0.20059d0*(rhostar_mix)**2.d0
      j6= Afact/Bfact**2.d0
!----------------------------------------

!dj6dz is d(I2)/d(rho*_mix)
      dj6dz = (-0.3618d0&
     &                -0.3205d0*2.d0*rhostar_mix &
     &                +0.1078d0*3.d0*rhostar_mix**2.d0 ) &
     &               / Bfact**2.d0 &
     &        + Afact*2.d0/Bfact**3.d0 *0.5236d0

!------------------------------------------

! dj6dz2 is d2(I2)/d(rho*_mix)2 which is not required!

!      dj6dz2 = (      -0.3205d0*2.d0*&
!     &                +0.1078d0*6.d0*rhostar_mix ) &
!     &               / Bfact**2.d0 &
!     &        +2.d0*(-0.3618d0 &
!     &                -0.3205d0*2.d0*rhostar_mix &
!     &                +0.1078d0*3.d0*rhostar_mix**2.d0 ) &
!     &               /Bfact**3.d0 *2.d0*0.5236d0 &
!     &        + Afact*6.d0/Bfact**4.d0 *(0.5236d0)**2.d0

!--------------------------------------------------

!kddd is the integral I3

      kddd= (1.d0+0.62378d0*rhostar_mix &
     &                -0.11658d0*(rhostar_mix)**2.d0 ) &
     &               / Cfact

!---------------------------------------------------
 
! dk_dz is d(I3)/d(rho*_mix)

      dk_dz = 1.d0/Cfact * (0.62378d0&
     &                -0.11658d0*2.d0*rhostar_mix &
     &                -kddd*(- 0.59056d0 &
     &                       +0.20059d0*2.d0*rhostar_mix))
      dk_dz=rhostar_mix*dk_dz+kddd
!---------------------------------------------------
! dk_dz2 is d2(I3)/d(rho*_mix)2

!      dk_dz2= 1.d0/Cfact * (-0.11658d0*2.d0 &
!     &                      -0.20059d0*2.d0*kddd &
!     &                      -2.d0*dk_dz*(- 0.59056d0 &
!     &                       +0.20059d0*2.d0*rhostar_mix))

!----------------------------------------------------
      dd_sum = 0.d0  ! summation in A2
      dd_su3 = 0.d0  ! summation in A3
      DO i=1,ncp
        DO j=1,ncp
          dd_sum = dd_sum+ x(i)*x(j)*m(i)*m(j) &
     &             *dhs(i)**3.d0*dhs(j)**3.d0  &
     &             /(0.5d0*(dhs(i)+dhs(j)))**3.d0 &
     &             *xp(i)*xp(j)*dd_my2(i)*dd_my2(j)
          DO k=1,ncp
            dd_su3=dd_su3+x(i)*x(j)*x(k) &
     &         *m(i)*m(j)*m(k) &
     &         *dhs(i)**3.d0*dhs(j)**3.d0*dhs(k)**3.d0 &
     &         /(0.5d0*(dhs(i)+dhs(j))) &
     &         /(0.5d0*(dhs(j)+dhs(k))) &
     &         /(0.5d0*(dhs(i)+dhs(k))) &
     &         *xp(i)*xp(j)*xp(k) &
     &         *dd_my2(i)*dd_my2(j)*dd_my2(k)
          END DO
        END DO
      END DO
!------------------------------------------------------
      f2_dip = -2.d0/9.d0*PI*rho*dd_sum*j6                  !A2/NkT
!
      f3_dip= 5.d0/162.d0*PI**2.d0 *rho**2.d0 * dd_su3*kddd !A3/NkT
!------------------------------------------------------

!fdd2dz is d(f2_dip)/d(rho*_mix)
! sig3_mix converts rho to rhostar_mix

      fdd2dz = -2.d0/9.d0*PI*dd_sum*(dj6dz*rhostar_mix+j6)/sig3_mix 

!fdd3dz is d(f3_dip)/d(rho*_mix)

      fdd3dz = 5.d0/162.d0*PI**2.d0 *dd_su3 &
     &           *(dk_dz*rhostar_mix**2 + 2.d0*rhostar_mix*kddd)/(sig3_mix**2)

      IF ((DABS(f2_dip).GT.1.d-100).AND.(DABS(f3_dip).GT.1.d-100)) THEN
        Apolar= f2_dip*( 1.d0 / (1.d0-f3_dip/f2_dip) )
! fdd_dz  = d(Apolar/NkT)/d(rho*_mix)
        fdd_dz = Apolar &
     &            * (  fdd2dz / f2_dip &
     &                  +  (fdd3dz-f3_dip/f2_dip*fdd2dz) &
     &                       / (f2_dip - f3_dip))
        zpolar = rhostar_mix*fdd_dz
      ENDIF
!-------------------------------------------------------
!
      DO k=1,ncp
      
! d(I2)/dx(k) = d(I2)/d(rho*_mix) * d(rho*_mix)/d(sig3_mix) * d(sig3_mix)/dx(k)

	  dj6dx(k) = ((-0.3618d0&
     &                -0.3205d0*2.d0*rhostar_mix &
     &                +0.1078d0*3.d0*rhostar_mix**2.d0) &
     &               / Bfact**2.d0 &
     &        + Afact*2.d0/Bfact**3.d0 *0.5236d0)*rho*dsig3_mix_dx(k)

!d(I3)/dx(k)

      dkdd_dx(k) = (1.d0/Cfact * (0.62378d0 &
     &                -0.11658d0*2.d0*rhostar_mix &
     &                -kddd*(- 0.59056d0 &
     &                       +0.20059d0*2.d0*rhostar_mix)) &
     &              )*rho*dsig3_mix_dx(k)


        dd_dx(k)  = 0.d0
        dd3_dx(k) = 0.d0
        DO i = 1,ncp
          dd_dx(k)  = dd_dx(k) + 2.d0*x(i)*m(i)*m(k) &
     &               *dhs(i)**3.d0*dhs(k)**3.d0 &
     &               /(0.5d0*(dhs(i)+dhs(k)))**3.d0 &
     &               *xp(i)*xp(k)*dd_my2(i)*dd_my2(k)
          DO j=1,ncp
            dd3_dx(k) = dd3_dx(k) &
     &         + 3.d0* x(i)*x(j) &
     &         *m(i)*m(j)*m(k) &
     &         *dhs(i)**3.d0*dhs(j)**3.d0*dhs(k)**3.d0 &
     &         /(0.5d0*(dhs(i)+dhs(j))) &
     &         /(0.5d0*(dhs(j)+dhs(k))) &
     &         /(0.5d0*(dhs(i)+dhs(k))) &
     &         *xp(i)*xp(j)*xp(k) &
     &         *dd_my2(i)*dd_my2(j)*dd_my2(k)
          END DO
        END DO

       m2_ddq(k)= -2.d0/9.d0*PI*rho* (dd_dx(k)*j6 + dd_sum*dj6dx(k))
       m3_ddq(k)= 5.d0/162.d0*PI**2.d0 *rho**2.d0 &
     &               * (dd3_dx(k)*kddd + dd_su3*dkdd_dx(k))

       m_dd(k)= Apolar &
     &          * (  m2_ddq(k) / f2_dip &
     &                +  (m3_ddq(k)-f3_dip/f2_dip*m2_ddq(k)) &
     &                     / (f2_dip - f3_dip))
       END DO

	myres=0.d0
	do i=1, ncp
		myres=myres-x(i)* m_dd(i)
	enddo
	do i=1,ncp
        cmupolar(i)=myres+apolar+zpolar +m_dd(i)
	enddo
!
    gpolar=apolar+zpolar


if (1.gt.2) then

! Calculating chemical potential of component 1 when component 1 is infinitely dilute
! (only component 2 has a dipole moment)

	  sig3_mix_num = m(2)**2.0d0*dhs(2)**3.d0
	  sig3_mix_den = m(2)**2.0d0

      dd_sum = 0.d0  ! summation in A2
      dd_su3 = 0.d0  ! summation in A3
    
      dd_sum = m(2)**2.0d0*xp(2)**2.0d0*dd_my2(2)**2.0d0*(dhs(2))**3.d0 
                 
      dd_su3 = m(2)**3.0d0*xp(2)**3.0d0*dd_my2(2)**6.0d0*(dhs(2))**6.d0

      f2_dip = -2.d0/9.d0*PI*rho*dd_sum*j6                  !A2/NkT

      f3_dip= 5.d0/162.d0*PI**2.d0 *rho**2.d0 * dd_su3*kddd !A3/NkT



	  rhostar_mix = rho*dhs(2)*3.0d0
	  sig3_mix = dhs(2)**3.0d0

	  
! j6 is the integral I2

       Afact = (1.d0- 0.3618d0*rhostar_mix &
     &              - 0.3205d0*(rhostar_mix)**2.d0 &
     &              + 0.1078d0*(rhostar_mix)**3.d0 )
       Bfact = 1.d0 - 0.5236d0*rhostar_mix
       Cfact = 1.d0 - 0.59056d0*rhostar_mix &
     &              + 0.20059d0*(rhostar_mix)**2.d0
      j6= Afact/Bfact**2.d0


!kddd is the integral I3

      kddd= (1.d0+0.62378d0*rhostar_mix &
     &                -0.11658d0*(rhostar_mix)**2.d0 ) &
     &               / Cfact


!dj6dz is d(I2)/d(rho*_mix)
      dj6dz = (-0.3618d0&
     &                -0.3205d0*2.d0*rhostar_mix &
     &                +0.1078d0*3.d0*rhostar_mix**2.d0 ) &
     &               / Bfact**2.d0 &
     &        + Afact*2.d0/Bfact**3.d0 *0.5236d0


! dk_dz is d(I3)/d(rho*_mix)

      dk_dz = 1.d0/Cfact * (0.62378d0&
     &                -0.11658d0*2.d0*rhostar_mix &
     &                -kddd*(- 0.59056d0 &
     &                       +0.20059d0*2.d0*rhostar_mix))
      dk_dz=rhostar_mix*dk_dz+kddd


!fdd2dz is d(f2_dip)/d(rho*_mix)


      fdd2dz = -2.d0/9.d0*PI*dd_sum*(dj6dz*rhostar_mix+j6)/sig3_mix 


!fdd3dz is d(f3_dip)/d(rho*_mix)


      fdd3dz = 5.d0/162.d0*PI**2.d0 *dd_su3 &
     &           *(dk_dz*rhostar_mix**2 + 2.d0*rhostar_mix*kddd)/(sig3_mix**2)


      if ((dabs(f2_dip).gt.1.d-100).and.(dabs(f3_dip).gt.1.d-100)) then
        Apolar= f2_dip*( 1.d0 / (1.d0-f3_dip/f2_dip) )

! fdd_dz  = d(Apolar/NkT)/d(rho*_mix)

        fdd_dz = Apolar &
     &            * (  fdd2dz / f2_dip &
     &                  +  (fdd3dz-f3_dip/f2_dip*fdd2dz) &
     &                       / (f2_dip - f3_dip))
        zpolar = rhostar_mix*fdd_dz
	  end if
!----------------------

dsig3_mix_num_dx(1) = 2*m(1)*m(2)*((dhs(1) + dhs(2))/2.0d0)**3.0d0
dsig3_mix_den_dx(1) = 2*m(1)*m(2)

dsig3_mix_num_dx(2) = 2*m(2)**2.0d0*dhs(2)**3.0d0
dsig3_mix_den_dx(2) = 2*m(2)**2.0d0


dd_dx(1)=0.0d0

dd_dx(2)  = 2.d0*m(2)**2.0d0*dhs(2)**3.d0*xp(2)**2.0d0*dd_my2(2)**2.0d0
 
dd3_dx(1) = 0.0d0 

dd3_dx(2)  = 3.d0*m(2)**3.0d0*dhs(2)**6.d0*xp(2)**3.0d0*dd_my2(2)**3.0d0  


	do k=1,2

	  dsig3_mix_dx(k)= (sig3_mix_den*dsig3_mix_num_dx(k) &
					 -sig3_mix_num*dsig3_mix_den_dx(k))/sig3_mix_den**2
	end do


     DO k=1,ncp
      
! d(I2)/dx(k) = d(I2)/d(rho*_mix) * d(rho*_mix)/d(sig3_mix) * d(sig3_mix)/dx(k)

	  dj6dx(k) = ((-0.3618d0&
     &                -0.3205d0*2.d0*rhostar_mix &
     &                +0.1078d0*3.d0*rhostar_mix**2.d0) &
     &               / Bfact**2.d0 &
     &        + Afact*2.d0/Bfact**3.d0 *0.5236d0)*rho*dsig3_mix_dx(k)

!d(I3)/dx(k)

      dkdd_dx(k) = (1.d0/Cfact * (0.62378d0 &
     &                -0.11658d0*2.d0*rhostar_mix &
     &                -kddd*(- 0.59056d0 &
     &                       +0.20059d0*2.d0*rhostar_mix)) &
     &              )*rho*dsig3_mix_dx(k)

     

       m2_ddq(k)= -2.d0/9.d0*PI*rho* (dd_dx(k)*j6 + dd_sum*dj6dx(k))
       m3_ddq(k)= 5.d0/162.d0*PI**2.d0 *rho**2.d0 &
     &               * (dd3_dx(k)*kddd + dd_su3*dkdd_dx(k))

       m_dd(k)= Apolar &
     &          * (  m2_ddq(k) / f2_dip &
     &                +  (m3_ddq(k)-f3_dip/f2_dip*m2_ddq(k)) &
     &                     / (f2_dip - f3_dip))
       END DO
 

	myres=0.0d0
	do i=1, ncp
		myres=myres-x(i)* m_dd(i)
	enddo
	do i=1,ncp
        cmupolar(i)=myres+apolar+zpolar +m_dd(i)
	enddo

end if
       return
       end